@@ -28,9 +28,10 @@ static void r_asm_list(RAsm *a) {
 static int rasm_show_help() {
 	printf ("rasm2 [-e] [-o offset] [-a arch] [-s syntax] -d \"opcode\"|\"hexpairs\"|- [-f file ..]\n"
 		" -d           Disassemble from hexpair bytes\n"
+		" -D           Disassemble showing hexpair and opcode\n"
 		" -f           Read data from file\n"
 		" -F [in:out]  Specify input and/or output filters (att2intel, x86.pseudo, ...)\n"
-		" -o [offset]  Offset where this opcode is suposed to be\n"
+		" -o [offset]  Set start address for code (0x%08"PFMT64x")\n"
 		" -a [arch]    Set architecture plugin\n"
 		" -b [bits]    Set architecture bits\n"
 		" -s [syntax]  Select syntax (intel, att)\n"
@@ -41,18 +42,20 @@ static int rasm_show_help() {
 		" -e           Use big endian\n"
 		" -v           Show version information\n"
 		" If '-l' value is greater than output length, output is padded with nops\n"
-		" If the last argument is '-' reads from stdin\n");
+		" If the last argument is '-' reads from stdin\n", R_SYS_BASE);
 	return 0;
 }
 
-static int rasm_disasm(char *buf, ut64 offset, ut64 len, int ascii, int bin) {
+static int rasm_disasm(char *buf, ut64 offset, ut64 len, int ascii, int bin, int hex) {
 	struct r_asm_code_t *acode;
-	ut8 *data;
+	ut8 *data = NULL;
 	char *ptr = buf;
 	int ret = 0;
 	ut64 word = 0, clen = 0; 
 
 	if (bin) {
+		if (len<0)
+			return R_FALSE;
 		clen = len; // XXX
 		data = (ut8*)buf;
 	} else if (ascii) {
@@ -62,21 +65,33 @@ static int rasm_disasm(char *buf, ut64 offset, ut64 len, int ascii, int bin) {
 		for (; *ptr; ptr++)
 			if (*ptr!=' ' && *ptr!='\n' && *ptr!='\r')
 				if (!(++word%2)) clen++;
-		data = alloca (clen);
+		data = malloc (clen+1);
 		if (r_hex_str2bin (buf, data)==-1)
-			return 0;
+			goto beach;
 	}
 
 	if (!len || clen <= len)
 		len = clen;
 
-	r_asm_set_pc (a, offset);
-	if (!(acode = r_asm_mdisassemble (a, data, len)))
-		return 0;
-
-	printf ("%s\n", acode->buf_asm);
-	ret = acode->len;
-	r_asm_code_free (acode);
+	if (hex) {
+		RAsmOp op;
+		r_asm_set_pc (a, offset);
+		while (r_asm_disassemble (a, &op, data+ret, len-ret) != -1) {
+			printf ("0x%08"PFMT64x"  %d %12s %s\n", 
+				a->pc, op.inst_len, op.buf_hex, op.buf_asm);
+			ret += op.inst_len;
+			r_asm_set_pc (a, offset+ret);
+		}
+	} else {
+		r_asm_set_pc (a, offset);
+		if (!(acode = r_asm_mdisassemble (a, data, len)))
+			goto beach;
+		printf ("%s", acode->buf_asm);
+		ret = acode->len;
+		r_asm_code_free (acode);
+	}
+beach:
+	if (data && data != (ut8*)buf) free (data);
 	return ret;
 }
 
@@ -130,7 +145,7 @@ static int __lib_asm_dt(struct r_lib_plugin_t *pl, void *p, void *u) { return R_
 
 int main(int argc, char *argv[]) {
 	char *arch = NULL, *file = NULL, *filters = NULL;
-	ut64 offset = 0x8048000;
+	ut64 offset = R_SYS_BASE;
 	int dis = 0, ascii = 0, bin = 0, ret = 0, bits = 32, c;
 	ut64 len = 0, idx = 0;
 
@@ -143,9 +158,12 @@ int main(int argc, char *argv[]) {
 	if (argc<2)
 		return rasm_show_help ();
 
-	r_asm_use (a, "x86"); // XXX: do not harcode default arch
-	while ((c = getopt (argc, argv, "Ceva:b:s:do:Bl:hLf:F:")) != -1) {
+	r_asm_use (a, R_SYS_ARCH);
+	while ((c = getopt (argc, argv, "DCeva:b:s:do:Bl:hLf:F:")) != -1) {
 		switch (c) {
+		case 'D':
+			dis = 2;
+			break;
 		case 'f':
 			file = optarg;
 			break;
@@ -229,13 +247,14 @@ int main(int argc, char *argv[]) {
 				eprintf ("WARNING: Cannot slurp more from stdin\n");
 			if (ret>=0)
 				buf[ret] = '\0';
-			if (dis) ret = rasm_disasm (buf, offset, len, ascii, bin);
+			if (dis)
+				ret = rasm_disasm (buf, offset, len, ascii, bin, dis-1);
 			else ret = rasm_asm (buf, offset, len, bin);
 		} else {
 			content = r_file_slurp (file, &length);
 			if (content) {
 				content[length] = '\0';
-				if (dis) ret = rasm_disasm (content, offset, len, ascii, bin);
+				if (dis) ret = rasm_disasm (content, offset, len, ascii, bin, dis-1);
 				else ret = rasm_asm (content, offset, len, bin);
 				free (content);
 			} else eprintf ("Cannot open file %s\n", file);
@@ -248,7 +267,7 @@ int main(int argc, char *argv[]) {
 				if ((!bin || !dis) && feof (stdin))
 					break;
 				if (!bin || !dis) buf[strlen (buf)-1]='\0';
-				if (dis) ret = rasm_disasm (buf, offset, len, ascii, bin);
+				if (dis) ret = rasm_disasm (buf, offset, len, ascii, bin, dis-1);
 				else ret = rasm_asm (buf, offset, len, bin);
 				idx += ret;
 				offset += ret;
@@ -261,7 +280,7 @@ int main(int argc, char *argv[]) {
 			}
 			return idx;
 		}
-		if (dis) ret = rasm_disasm (argv[optind], offset, len, ascii, bin);
+		if (dis) ret = rasm_disasm (argv[optind], offset, len, ascii, bin, dis-1);
 		else ret = rasm_asm (argv[optind], offset, len, bin);
 		if (!ret) eprintf ("invalid\n");
 		return ret;
