@@ -6,7 +6,7 @@ static int flags = 0;
 
 static int format_output (char mode, ut64 n);
 static int help ();
-static int rax (char *str, int last);
+static int rax (char *str, int len, int last);
 static int use_stdin ();
 
 static int format_output (char mode, ut64 n) {
@@ -56,17 +56,19 @@ static int help () {
 		"  -b    binstr -> bin     ;  rax2 -b 01000101 01110110\n"
 		"  -s    hexstr -> bin     ;  rax2 -s 43 4a 50\n"
 		"  -S    bin -> hexstr     ;  rax2 -S C  J  P\n"
-		"  -V    version           ;  rax2 -V\n"
+		"  -v    version           ;  rax2 -V\n"
 		"  -x    hash string       ;  rax2 -x linux osx\n"
 		"  -k    keep base         ;  rax2 -k 33+3 -> 36\n"
 		"  -h    help              ;  rax2 -h\n");
 	return R_TRUE;
 }
 
-static int rax (char *str, int last) {
+static int rax (char *str, int len, int last) {
 	float f;
 	char *p, *buf, out_mode = '0';
 	int i;
+	if (!len)
+		len = strlen (str);
 
 	if (*str=='-') {
 		switch (str[1]) {
@@ -88,7 +90,7 @@ static int rax (char *str, int last) {
 		case 'k':
 			flags ^= 32;
 			break;
-		case 'V':
+		case 'v':
 			printf ("rax2 v"R2_VERSION"\n");
 			break;
 		case '\0':
@@ -116,9 +118,9 @@ static int rax (char *str, int last) {
 		free (buf);
 		return R_TRUE;
 	}
-	if (flags & 4) {
-		for (i=0; str[i]; i++)
-			printf ("%02x", str[i]);
+	if (flags & 4) { // -S
+		for (i=0; i<len; i++)
+			printf ("%02x", (ut8)str[i]);
 		printf ("\n");
 		return R_TRUE;
 	}
@@ -177,10 +179,16 @@ static int rax (char *str, int last) {
 static int use_stdin () {
 	char buf[4096]; // TODO: remove this limit
 	while (!feof (stdin)) {
-		fgets (buf, sizeof (buf), stdin);
+		int n = read (0, buf, sizeof (buf));
+		if (n<1) break;
+		buf[n] = 0;
+		//fgets (buf, sizeof (buf), stdin);
 		if (feof (stdin)) break;
-		buf[strlen (buf)-1] = '\0';
-		if (!rax (buf, 0)) break;
+		if ((flags & 4) && strlen (buf) < sizeof (buf)) // -S
+			buf[strlen (buf)] = '\0';
+		else
+			buf[strlen (buf)-1] = '\0';
+		if (!rax (buf, n, 0)) break;
 	}
 	return 0;
 }
@@ -190,6 +198,6 @@ int main (int argc, char **argv) {
 	if (argc == 1)
 		return use_stdin ();
 	for (i=1; i<argc; i++)
-		rax (argv[i], (i+1)==argc);
+		rax (argv[i], 0, (i+1)==argc);
 	return 0;
 }
