@@ -37,6 +37,7 @@ static void get_strings_range(RBinArch *arch, RList *list, int min, ut64 from, u
 			str[matches] = '\0';
 			ptr->offset = i-matches;
 			ptr->rva = ptr->offset-from+scnrva;
+			//HACK if (scnrva) ptr->rva = ptr->offset-from+scnrva; else ptr->rva = ptr->offset;
 			ptr->size = matches+1;
 			ptr->ordinal = ctr;
 			// copying so many bytes here..
@@ -174,6 +175,7 @@ static int r_bin_extract(RBin *bin, int idx) {
 	ut8 *buf;
 	if (bin->curxtr && bin->curxtr->extract)
 		return bin->curxtr->extract (bin, idx);
+	//free (bin->curarch.file);
 	bin->curarch.file = strdup (bin->file);
 	if (!(buf = (ut8*)r_file_slurp (bin->file, &bin->curarch.size))) 
 		return 0;
@@ -408,9 +410,9 @@ R_API void r_bin_list_archs(RBin *bin) {
 	int i;
 	for (i = 0; i < bin->narch; i++)
 		if (r_bin_select_idx (bin, i) && bin->curarch.info)
-			printf ("%03i %s %s_%i (%s)\n", i, bin->curarch.file,
-					bin->curarch.info->arch, bin->curarch.info->bits,
-					bin->curarch.info->machine);
+			printf ("%03i 0x%08"PFMT64x" %s_%i %s\n", i, 
+				bin->curarch.offset, bin->curarch.info->arch,
+				bin->curarch.info->bits, bin->curarch.info->machine);
 }
 
 R_API void r_bin_set_user_ptr(RBin *bin, void *user) {
@@ -448,6 +450,7 @@ R_API RBinObj *r_bin_get_object(RBin *bin, int flags) {
 		for (i=0; i<R_BIN_SYM_LAST; i++)
 			obj->binsym[i] = r_bin_get_sym (bin, i);
 		obj->baddr = r_bin_get_baddr (bin);
+		obj->info = r_bin_get_info (bin);
 	}
 	return obj;
 }
@@ -460,3 +463,10 @@ R_API void r_bin_object_free(RBinObj *obj) {
 R_API RList* /*<RBinClass>*/r_bin_get_classes(RBin *bin) {
 	return bin->curarch.classes;
 }
+
+R_API ut64 r_bin_get_offset (RBin *bin) {
+	ut64 offset = bin->curarch.offset;
+	if (offset>0x1000) // XXX HACK
+		offset -= 0x1000;
+	return offset;
+}
