@@ -21,14 +21,12 @@ static inline int __strnlen(const char *str, int len) {
 static int Elf_(r_bin_elf_init_ehdr)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	ut8 e_ident[16];
 	int len;
-
 	if (r_buf_read_at (bin->b, 0, e_ident, 16) == -1) {
 		eprintf ("Error: read (magic)\n");
 		return R_FALSE;
 	}
-	if (e_ident[EI_DATA] == ELFDATA2MSB)
-		bin->endian = LIL_ENDIAN;
-	else bin->endian = !LIL_ENDIAN;
+	bin->endian = (e_ident[EI_DATA] == ELFDATA2MSB)?
+		LIL_ENDIAN: !LIL_ENDIAN;
 	len = r_buf_fread_at (bin->b, 0, (ut8*)&bin->ehdr,
 #if R_BIN_ELF64
 		bin->endian?"16c2SI3LI6S":"16c2si3li6s",
@@ -47,20 +45,21 @@ static int Elf_(r_bin_elf_init_ehdr)(struct Elf_(r_bin_elf_obj_t) *bin) {
 
 static int Elf_(r_bin_elf_init_phdr)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	int phdr_size, len;
-
-	if (bin->ehdr.e_phnum == 0)
+	if (bin->ehdr.e_phnum == 0) {
 		return R_FALSE;
+	}
+	if (bin->phdr) return R_TRUE;
 	phdr_size = bin->ehdr.e_phnum * sizeof (Elf_(Phdr));
 	if ((bin->phdr = (Elf_(Phdr) *)malloc (phdr_size)) == NULL) {
 		perror ("malloc (phdr)");
 		return R_FALSE;
 	}
 	len = r_buf_fread_at (bin->b, bin->ehdr.e_phoff, (ut8*)bin->phdr,
-#if R_BIN_ELF64
+		#if R_BIN_ELF64
 		bin->endian?"2I6L":"2i6l",
-#else
+		#else
 		bin->endian?"8I":"8i",
-#endif
+		#endif
 		bin->ehdr.e_phnum);
 	if (len == -1) {
 		eprintf ("Error: read (phdr)\n");
@@ -71,8 +70,8 @@ static int Elf_(r_bin_elf_init_phdr)(struct Elf_(r_bin_elf_obj_t) *bin) {
 }
 
 static int Elf_(r_bin_elf_init_shdr)(struct Elf_(r_bin_elf_obj_t) *bin) {
-	int shdr_size, len;
-	
+	int len, shdr_size;
+	if (bin->shdr) return R_TRUE;
 	shdr_size = bin->ehdr.e_shnum * sizeof (Elf_(Shdr));
 	if ((bin->shdr = (Elf_(Shdr) *)malloc (shdr_size)) == NULL) {
 		perror ("malloc (shdr)");
@@ -95,8 +94,7 @@ static int Elf_(r_bin_elf_init_shdr)(struct Elf_(r_bin_elf_obj_t) *bin) {
 
 static int Elf_(r_bin_elf_init_strtab)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	int sz;
-	if (!bin->shdr)
-		return R_FALSE;
+	if (bin->strtab) return R_FALSE;
 	bin->shstrtab_section =
 	bin->strtab_section = &bin->shdr[bin->ehdr.e_shstrndx];
 	bin->shstrtab_size =
@@ -135,7 +133,7 @@ static ut64 Elf_(r_bin_elf_get_section_size)(struct Elf_(r_bin_elf_obj_t) *bin,
 
 static int Elf_(r_bin_elf_init_strtab)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	ut64 size;
-	if (!bin->shdr)
+	if (!bin->shstrtab)
 		return  R_FALSE;
 	bin->shstrtab_section = &bin->shdr[bin->ehdr.e_shstrndx];
 	bin->shstrtab_size = size = 4096; //bin->shstrtab_section->sh_size;
@@ -185,8 +183,9 @@ static int Elf_(r_bin_elf_init)(struct Elf_(r_bin_elf_obj_t) *bin) {
 		eprintf ("Warning: File is not ELF\n");
 		return R_FALSE;
 	}
-	if (!Elf_(r_bin_elf_init_phdr) (bin))
-		eprintf ("Warning: Cannot initialize program headers\n");
+	Elf_(r_bin_elf_init_phdr) (bin);
+	//if (!Elf_(r_bin_elf_init_phdr) (bin))
+		//eprintf ("Warning: Cannot initialize program headers\n");
 	if (!Elf_(r_bin_elf_init_shdr) (bin))
 		eprintf ("Warning: Cannot initialize section headers\n");
 	if (!Elf_(r_bin_elf_init_strtab) (bin))
@@ -223,7 +222,7 @@ static ut64 Elf_(r_bin_elf_get_section_addr)(struct Elf_(r_bin_elf_obj_t) *bin,
 }
 
 static ut64 Elf_(get_import_addr)(struct Elf_(r_bin_elf_obj_t) *bin, int sym) {
-	Elf_(Rel) *rel;
+	Elf_(Rel) *rel = NULL;
 	Elf_(Addr) plt_sym_addr;
 	ut64 got_addr, got_offset;
 	int i, j, k, tsize, len;
@@ -243,6 +242,7 @@ static ut64 Elf_(get_import_addr)(struct Elf_(r_bin_elf_obj_t) *bin, int sym) {
 			tsize = sizeof (Elf_(Rela));
 		else continue;
 
+		free (rel);
 		if ((rel = (Elf_(Rel) *)malloc ((int)(bin->shdr[i].sh_size / tsize) * sizeof (Elf_(Rel)))) == NULL) {
 			perror ("malloc (rel)");
 			return -1;
@@ -257,6 +257,7 @@ static ut64 Elf_(get_import_addr)(struct Elf_(r_bin_elf_obj_t) *bin, int sym) {
 					1);
 			if (len == -1) {
 				eprintf ("Error: read (rel)\n");
+				free (rel);
 				return -1;
 			}
 		}
@@ -265,14 +266,16 @@ static ut64 Elf_(get_import_addr)(struct Elf_(r_bin_elf_obj_t) *bin, int sym) {
 				if (r_buf_read_at (bin->b, rel[k].r_offset-got_addr+got_offset,
 							(ut8*)&plt_sym_addr, sizeof (Elf_(Addr))) == -1) {
 					eprintf ("Error: read (got)\n");
-					return -1;
+					return UT64_MAX;
 				}
+				free (rel);
 				return (ut64)(plt_sym_addr - 6);
 			}
 		}
 		break;
 	}
-	return -1;
+	free (rel);
+	return UT64_MAX;
 }
 
 ut64 Elf_(r_bin_elf_get_baddr)(struct Elf_(r_bin_elf_obj_t) *bin) {
@@ -308,7 +311,7 @@ ut64 Elf_(r_bin_elf_get_fini_offset)(struct Elf_(r_bin_elf_obj_t) *bin) {
 		eprintf ("Error: read (entry)\n");
 		return 0;
 	}
-	if (buf[0] == 0x68) { // push // x86/32 only
+	if (*buf == 0x68) { // push // x86/32 only
 		memmove (buf, buf+1, 4);
 		return (ut64)((int)(buf[0]+(buf[1]<<8)+(buf[2]<<16)+(buf[3]<<24)))-bin->baddr;
 	}
@@ -316,6 +319,15 @@ ut64 Elf_(r_bin_elf_get_fini_offset)(struct Elf_(r_bin_elf_obj_t) *bin) {
 }
 
 ut64 Elf_(r_bin_elf_get_entry_offset)(struct Elf_(r_bin_elf_obj_t) *bin) {
+	ut64 entry = (ut64) bin->ehdr.e_entry;
+	if (entry == 0LL) {
+		entry = Elf_(r_bin_elf_get_section_offset)(bin, ".init.text");
+		if (entry != UT64_MAX) return entry;
+		entry = Elf_(r_bin_elf_get_section_offset)(bin, ".text");
+		if (entry != UT64_MAX) return entry;
+		entry = Elf_(r_bin_elf_get_section_offset)(bin, ".init");
+		if (entry != UT64_MAX) return entry;
+	}
 	if (bin->ehdr.e_entry < bin->baddr)
 		return bin->ehdr.e_entry;
 	return bin->ehdr.e_entry - bin->baddr; 
@@ -391,6 +403,12 @@ char* Elf_(r_bin_elf_get_data_encoding)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	}
 }
 
+int Elf_(r_bin_elf_has_va)(struct Elf_(r_bin_elf_obj_t) *bin) {
+	ut32 e_type = (ut32)bin->ehdr.e_type; // cast to avoid warn in iphone-gcc, must be ut16
+	//if (bin->ehdr.e_phnum == 0)
+	return (e_type == ET_REL)? 0: 1;
+}
+
 // TODO: do not strdup here
 char* Elf_(r_bin_elf_get_arch)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	switch (bin->ehdr.e_machine) {
@@ -585,8 +603,11 @@ char *Elf_(r_bin_elf_get_rpath)(struct Elf_(r_bin_elf_obj_t) *bin) {
 		return NULL;
 	for (i = 0; i < bin->ehdr.e_phnum; i++)
 		if (bin->phdr[i].p_type == PT_DYNAMIC) {
+			free (dyn); // TODO: reuse dyn allocation
 			if (!(dyn = malloc (1+bin->phdr[i].p_filesz))) {
 				perror ("malloc (dyn)");
+				free (ret);
+				free (dyn);
 				return NULL;
 			}
 			ndyn = (int)(bin->phdr[i].p_filesz / sizeof (Elf_(Dyn)));
@@ -599,6 +620,7 @@ char *Elf_(r_bin_elf_get_rpath)(struct Elf_(r_bin_elf_obj_t) *bin) {
 					ndyn);
 			if (len  == -1) {
 				eprintf ("Error: read (dyn)\n");
+				free (ret);
 				free (dyn);
 				return NULL;
 			}
@@ -609,6 +631,7 @@ char *Elf_(r_bin_elf_get_rpath)(struct Elf_(r_bin_elf_obj_t) *bin) {
 				}
 			for (j = 0; j < ndyn; j++)
 				if (dyn[j].d_tag == DT_RPATH || dyn[j].d_tag == DT_RUNPATH) {
+					free (ret);
 					if ((ret = malloc (ELF_STRING_LENGTH)) == NULL) {
 						perror ("malloc (rpath)");
 						free (dyn);
@@ -623,9 +646,9 @@ char *Elf_(r_bin_elf_get_rpath)(struct Elf_(r_bin_elf_obj_t) *bin) {
 					}
 					break;
 				}
-			free (dyn);
 			break;
 		}
+	free (dyn);
 	return ret;
 }
 
@@ -656,7 +679,7 @@ struct r_bin_elf_reloc_t* Elf_(r_bin_elf_get_relocs)(struct Elf_(r_bin_elf_obj_t
 				eprintf ("Error: read (syms strtab)\n");
 				return NULL;
 			}
-			if ((sym = (Elf_(Sym) *)malloc (1+bin->shdr[i].sh_size)) == NULL) {
+			if ((sym = (Elf_(Sym) *)malloc (1+bin->shdr[i].sh_size)) == NULL) { // LEAKS
 				perror ("malloc (syms)");
 				return NULL;
 			}
@@ -707,6 +730,7 @@ struct r_bin_elf_reloc_t* Elf_(r_bin_elf_get_relocs)(struct Elf_(r_bin_elf_obj_t
 			perror ("malloc (reloc)");
 			return NULL;
 		}
+		if (sym)
 		for (j =  0; j < nrel; j++) {
 			idx = ELF_R_SYM (rel[j].r_info);
 			if (idx < nsym) {
@@ -797,8 +821,6 @@ struct r_bin_elf_section_t* Elf_(r_bin_elf_get_sections)(struct Elf_(r_bin_elf_o
 	struct r_bin_elf_section_t *ret = NULL;
 	int i, nidx;
 	
-	if (!bin->shdr)
-		return NULL;
 	if ((ret = malloc ((bin->ehdr.e_shnum + 1) * sizeof (struct r_bin_elf_section_t))) == NULL)
 		return NULL;
 	for (i = 0; i < bin->ehdr.e_shnum; i++) {
@@ -832,6 +854,7 @@ struct r_bin_elf_symbol_t* Elf_(r_bin_elf_get_symbols)(struct Elf_(r_bin_elf_obj
 	if (!bin->shdr || bin->ehdr.e_shnum == 0)
 		return NULL;
 	if (bin->ehdr.e_type == ET_REL) {
+		// XXX: we must obey shndx here
 		if ((sym_offset = Elf_(r_bin_elf_get_section_offset)(bin, ".text")) == -1)
 			sym_offset = 0;
 		if ((data_offset = Elf_(r_bin_elf_get_section_offset)(bin, ".rodata")) == -1)
@@ -859,9 +882,9 @@ struct r_bin_elf_symbol_t* Elf_(r_bin_elf_get_symbols)(struct Elf_(r_bin_elf_obj
 			nsym = (int)(bin->shdr[i].sh_size/sizeof (Elf_(Sym)));
 			if (r_buf_fread_at (bin->b, bin->shdr[i].sh_offset, (ut8*)sym,
 #if R_BIN_ELF64
-					bin->endian?"I2cS2L":"i2cs2l",
+					bin->endian? "I2cS2L": "i2cs2l",
 #else
-					bin->endian?"3I2cS":"3i2cs",
+					bin->endian? "3I2cS": "3i2cs",
 #endif
 					nsym) == -1) {
 				eprintf ("Error: read (sym)\n");
@@ -878,15 +901,27 @@ struct r_bin_elf_symbol_t* Elf_(r_bin_elf_get_symbols)(struct Elf_(r_bin_elf_obj
 					tsize = 16;
 				} else if (type == R_BIN_ELF_SYMBOLS && sym[k].st_shndx != STN_UNDEF &&
 						ELF_ST_TYPE(sym[k].st_info) != STT_SECTION && ELF_ST_TYPE(sym[k].st_info) != STT_FILE) {
-					toffset = (ut64)sym[k].st_value +
-						(ELF_ST_TYPE(sym[k].st_info) == STT_FUNC?sym_offset:data_offset);
+					int idx = sym[k].st_shndx;
 					tsize = sym[k].st_size;
+					toffset = (ut64)sym[k].st_value; //-sym_offset; // + (ELF_ST_TYPE(sym[k].st_info) == STT_FUNC?sym_offset:data_offset);
 				} else continue;
 				if ((ret = realloc (ret, (ret_ctr + 1) * sizeof (struct r_bin_elf_symbol_t))) == NULL) {
 					perror ("realloc (symbols|imports)");
 					return NULL;
 				}
-				ret[ret_ctr].offset = (toffset >= bin->baddr ? toffset -= bin->baddr : toffset);
+				{
+					int idx = sym[k].st_shndx;
+					if (idx>=0 && idx < bin->ehdr.e_shnum) {
+						if (bin->baddr && toffset>bin->baddr)
+							toffset -= bin->baddr;
+						else
+							toffset += bin->shdr[idx].sh_offset;
+					} else {
+						//eprintf ("orphan symbol %d %d %s\n", idx, STN_UNDEF, &strtab[sym[k].st_name] );
+						continue;
+					}
+				}
+				ret[ret_ctr].offset = toffset; //(toffset >= bin->baddr ? toffset -= bin->baddr : toffset);
 				ret[ret_ctr].size = tsize;
 				if (sym[k].st_name > strtab_section->sh_size) {
 					perror ("index out of strtab range\n");
@@ -928,7 +963,7 @@ struct r_bin_elf_symbol_t* Elf_(r_bin_elf_get_symbols)(struct Elf_(r_bin_elf_obj
 			}
 			if ((ret = realloc (ret, (ret_ctr + 1) * sizeof (struct r_bin_elf_symbol_t))) == NULL)
 				return NULL;
-			ret[ret_ctr].last = 1;
+			ret[ret_ctr].last = 1; // ugly dirty hack :D
 			break;
 		}
 	return ret;
@@ -960,8 +995,7 @@ struct r_bin_elf_field_t* Elf_(r_bin_elf_get_fields)(struct Elf_(r_bin_elf_obj_t
 }
 
 void* Elf_(r_bin_elf_free)(struct Elf_(r_bin_elf_obj_t)* bin) {
-	if (!bin)
-		return NULL;
+	if (!bin) return NULL;
 	free (bin->phdr);
 	free (bin->shdr);
 	free (bin->strtab);
@@ -974,7 +1008,7 @@ void* Elf_(r_bin_elf_free)(struct Elf_(r_bin_elf_obj_t)* bin) {
 struct Elf_(r_bin_elf_obj_t)* Elf_(r_bin_elf_new)(const char* file) {
 	struct Elf_(r_bin_elf_obj_t) *bin;
 	ut8 *buf;
-
+// TODO: use R_NEW0 here
 	if (!(bin = malloc (sizeof (struct Elf_(r_bin_elf_obj_t)))))
 		return NULL;
 	memset (bin, 0, sizeof (struct Elf_(r_bin_elf_obj_t)));
@@ -992,7 +1026,7 @@ struct Elf_(r_bin_elf_obj_t)* Elf_(r_bin_elf_new)(const char* file) {
 
 struct Elf_(r_bin_elf_obj_t)* Elf_(r_bin_elf_new_buf)(struct r_buf_t *buf) {
 	struct Elf_(r_bin_elf_obj_t) *bin;
-
+// TODO: use R_NEW here
 	if (!(bin = malloc (sizeof (struct Elf_(r_bin_elf_obj_t)))))
 		return NULL;
 	memset (bin, 0, sizeof (struct Elf_(r_bin_elf_obj_t)));
