@@ -123,13 +123,12 @@ static int PE_(r_bin_pe_parse_imports)(struct PE_(r_bin_pe_obj_t)* bin, struct r
 	return i;
 }
 
-static int PE_(r_bin_pe_init_hdr)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+static int PE_(r_bin_pe_init_hdr)(struct PE_(r_bin_pe_obj_t)* bin) {
 	if (!(bin->dos_header = malloc(sizeof(PE_(image_dos_header))))) {
-		perror("malloc (dos header)");
+		perror ("malloc (dos header)");
 		return R_FALSE;
 	}
-	if (r_buf_read_at(bin->b, 0, (ut8*)bin->dos_header, sizeof(PE_(image_dos_header))) == -1) {
+	if (r_buf_read_at (bin->b, 0, (ut8*)bin->dos_header, sizeof(PE_(image_dos_header))) == -1) {
 		eprintf("Error: read (dos header)\n");
 		return R_FALSE;
 	}
@@ -141,38 +140,37 @@ static int PE_(r_bin_pe_init_hdr)(struct PE_(r_bin_pe_obj_t)* bin)
 		perror("malloc (nt header)");
 		return R_FALSE;
 	}
-	if (r_buf_read_at(bin->b, bin->dos_header->e_lfanew,
+	if (r_buf_read_at (bin->b, bin->dos_header->e_lfanew,
 				(ut8*)bin->nt_headers, sizeof(PE_(image_nt_headers))) == -1) {
-		eprintf("Error: read (dos header)\n");
+		eprintf ("Error: read (dos header)\n");
 		return R_FALSE;
 	}
-	if (strncmp((char*)&bin->dos_header->e_magic, "MZ", 2) ||
-		strncmp((char*)&bin->nt_headers->Signature, "PE", 2))
+	if (strncmp ((char*)&bin->dos_header->e_magic, "MZ", 2) ||
+		strncmp ((char*)&bin->nt_headers->Signature, "PE", 2))
 		return R_FALSE;
 	return R_TRUE;
 }
 
-static int PE_(r_bin_pe_init_sections)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+static int PE_(r_bin_pe_init_sections)(struct PE_(r_bin_pe_obj_t)* bin) {
 	int sections_size = sizeof(PE_(image_section_header)) * bin->nt_headers->file_header.NumberOfSections;
 	if (sections_size > bin->size) {
-		eprintf("Invalid NumberOfSections value\n");
+		eprintf ("Invalid NumberOfSections value\n");
 		return R_FALSE;
 	}
-	if (!(bin->section_header = malloc(sections_size))) {
-		perror("malloc (section header)");
+	if (!(bin->section_header = malloc (sections_size))) {
+		perror ("malloc (section header)");
 		return R_FALSE;
 	}
-	if (r_buf_read_at(bin->b, bin->dos_header->e_lfanew + sizeof(PE_(image_nt_headers)),
+	if (r_buf_read_at (bin->b, bin->dos_header->e_lfanew + 4 + sizeof (PE_(image_file_header)) +
+				bin->nt_headers->file_header.SizeOfOptionalHeader,
 				(ut8*)bin->section_header, sections_size) == -1) {
-		eprintf("Error: read (import directory)\n");
+		eprintf ("Error: read (import directory)\n");
 		return R_FALSE;
 	}
 	return R_TRUE;
 }
 
-static int PE_(r_bin_pe_init_imports)(struct PE_(r_bin_pe_obj_t) *bin)
-{
+static int PE_(r_bin_pe_init_imports)(struct PE_(r_bin_pe_obj_t) *bin) {
 	PE_(image_data_directory) *data_dir_import = &bin->nt_headers->optional_header.DataDirectory[PE_IMAGE_DIRECTORY_ENTRY_IMPORT];
 	PE_(image_data_directory) *data_dir_delay_import = &bin->nt_headers->optional_header.DataDirectory[PE_IMAGE_DIRECTORY_ENTRY_DELAY_IMPORT];
 	PE_DWord import_dir_offset = PE_(r_bin_pe_rva_to_offset)(bin, data_dir_import->VirtualAddress);
@@ -239,7 +237,7 @@ static int PE_(r_bin_pe_init)(struct PE_(r_bin_pe_obj_t)* bin)
 		return R_FALSE;
 	}
 	if (!PE_(r_bin_pe_init_sections)(bin)) {
-		eprintf("Warning: Cannot initalize sections\n");
+		eprintf("Warning: Cannot initialize sections\n");
 		return R_FALSE;
 	}
 	PE_(r_bin_pe_init_imports)(bin);
@@ -410,36 +408,37 @@ struct r_bin_pe_import_t* PE_(r_bin_pe_get_imports)(struct PE_(r_bin_pe_obj_t) *
 	return imports;
 }
 
-struct r_bin_pe_lib_t* PE_(r_bin_pe_get_libs)(struct PE_(r_bin_pe_obj_t) *bin)
-{
+struct r_bin_pe_lib_t* PE_(r_bin_pe_get_libs)(struct PE_(r_bin_pe_obj_t) *bin) {
 	struct r_bin_pe_lib_t *libs = NULL;
 	int import_dirs_count = PE_(r_bin_pe_get_import_dirs_count)(bin);
 	int delay_import_dirs_count = PE_(r_bin_pe_get_delay_import_dirs_count)(bin);
-	int i, j;
+	int i, j = 0;
 	
 	if ((libs = malloc((import_dirs_count + delay_import_dirs_count + 2) * sizeof(struct r_bin_pe_string_t))) == NULL) {
 		perror("malloc (libs)");
 		return NULL;
 	}
-	for (i = j = 0; i < import_dirs_count; i++, j++) {
-		if (r_buf_read_at(bin->b, PE_(r_bin_pe_rva_to_offset)(bin, bin->import_directory[i].Name),
-				(ut8*)libs[j].name, PE_STRING_LENGTH) == -1) {
-			eprintf("Error: read (libs - import dirs)\n");
-			return NULL;
+	if (bin->import_directory) {
+		for (i = j = 0; i < import_dirs_count; i++, j++) {
+			if (r_buf_read_at(bin->b, PE_(r_bin_pe_rva_to_offset)(bin, bin->import_directory[i].Name),
+					(ut8*)libs[j].name, PE_STRING_LENGTH) == -1) {
+				eprintf("Error: read (libs - import dirs)\n");
+				return NULL;
+			}
+			if (PE_(r_bin_pe_rva_to_offset)(bin, bin->import_directory[i].Characteristics) == 0 &&
+				PE_(r_bin_pe_rva_to_offset)(bin, bin->import_directory[i].FirstThunk) == 0)
+				break;
 		}
-		if (PE_(r_bin_pe_rva_to_offset)(bin, bin->import_directory[i].Characteristics) == 0 &&
-			PE_(r_bin_pe_rva_to_offset)(bin, bin->import_directory[i].FirstThunk) == 0)
-			break;
-	}
-	for (i = 0; i < delay_import_dirs_count; i++, j++) {
-		if (r_buf_read_at(bin->b, PE_(r_bin_pe_rva_to_offset)(bin, bin->delay_import_directory[i].Name),
-				(ut8*)libs[j].name, PE_STRING_LENGTH) == -1) {
-			eprintf("Error: read (libs - delay import dirs)\n");
-			return NULL;
+		for (i = 0; i < delay_import_dirs_count; i++, j++) {
+			if (r_buf_read_at(bin->b, PE_(r_bin_pe_rva_to_offset)(bin, bin->delay_import_directory[i].Name),
+					(ut8*)libs[j].name, PE_STRING_LENGTH) == -1) {
+				eprintf("Error: read (libs - delay import dirs)\n");
+				return NULL;
+			}
+			if (PE_(r_bin_pe_rva_to_offset)(bin, bin->delay_import_directory[i].DelayImportNameTable) == 0 &&
+				PE_(r_bin_pe_rva_to_offset)(bin, bin->delay_import_directory[i].DelayImportAddressTable) == 0)
+				break;
 		}
-		if (PE_(r_bin_pe_rva_to_offset)(bin, bin->delay_import_directory[i].DelayImportNameTable) == 0 &&
-			PE_(r_bin_pe_rva_to_offset)(bin, bin->delay_import_directory[i].DelayImportAddressTable) == 0)
-			break;
 	}
 	for (i = 0; i < j; i++) {
 		libs[i].name[PE_STRING_LENGTH-1] = '\0';
@@ -449,13 +448,12 @@ struct r_bin_pe_lib_t* PE_(r_bin_pe_get_libs)(struct PE_(r_bin_pe_obj_t) *bin)
 	return libs;
 }
 
-int PE_(r_bin_pe_get_image_size)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+int PE_(r_bin_pe_get_image_size)(struct PE_(r_bin_pe_obj_t)* bin) {
 	return bin->nt_headers->optional_header.SizeOfImage;
 }
 
-char* PE_(r_bin_pe_get_machine)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+// TODO: make it const! like in elf
+char* PE_(r_bin_pe_get_machine)(struct PE_(r_bin_pe_obj_t)* bin) {
 	char *machine;
 
 	switch (bin->nt_headers->file_header.Machine) {
@@ -546,12 +544,11 @@ char* PE_(r_bin_pe_get_machine)(struct PE_(r_bin_pe_obj_t)* bin)
 	default:
 		machine = strdup("unknown");
 	}
-
 	return machine;
 }
 
-char* PE_(r_bin_pe_get_os)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+// TODO: make it const! like in elf
+char* PE_(r_bin_pe_get_os)(struct PE_(r_bin_pe_obj_t)* bin) {
 	char *os;
 
 	switch (bin->nt_headers->optional_header.Subsystem) {
@@ -561,29 +558,29 @@ char* PE_(r_bin_pe_get_os)(struct PE_(r_bin_pe_obj_t)* bin)
 	case PE_IMAGE_SUBSYSTEM_WINDOWS_GUI:
 	case PE_IMAGE_SUBSYSTEM_WINDOWS_CUI:
 	case PE_IMAGE_SUBSYSTEM_WINDOWS_CE_GUI:
-		os = strdup("windows");
+		os = strdup ("windows");
 		break;
 	case PE_IMAGE_SUBSYSTEM_POSIX_CUI:
-		os = strdup("posix");
+		os = strdup ("posix");
 		break;
 	case PE_IMAGE_SUBSYSTEM_EFI_APPLICATION:
 	case PE_IMAGE_SUBSYSTEM_EFI_BOOT_SERVICE_DRIVER:
 	case PE_IMAGE_SUBSYSTEM_EFI_RUNTIME_DRIVER:
 	case PE_IMAGE_SUBSYSTEM_EFI_ROM:
-		os = strdup("efi");
+		os = strdup ("efi");
 		break;
 	case PE_IMAGE_SUBSYSTEM_XBOX:
-		os = strdup("xbox");
+		os = strdup ("xbox");
 		break;
 	default:
-		os = strdup("unknown");
+		// XXX: this is unknown
+		os = strdup ("windows");
 	}
-
 	return os;
 }
 
-char* PE_(r_bin_pe_get_class)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+// TODO: make it const
+char* PE_(r_bin_pe_get_class)(struct PE_(r_bin_pe_obj_t)* bin) {
 	char *class;
 
 	switch (bin->nt_headers->optional_header.Magic) {
@@ -599,8 +596,7 @@ char* PE_(r_bin_pe_get_class)(struct PE_(r_bin_pe_obj_t)* bin)
 	return class;
 }
 
-int PE_(r_bin_pe_get_bits)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+int PE_(r_bin_pe_get_bits)(struct PE_(r_bin_pe_obj_t)* bin) {
 	int bits;
 
 	switch (bin->nt_headers->optional_header.Magic) {
@@ -616,23 +612,21 @@ int PE_(r_bin_pe_get_bits)(struct PE_(r_bin_pe_obj_t)* bin)
 	return bits;
 }
 
-int PE_(r_bin_pe_get_section_alignment)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+int PE_(r_bin_pe_get_section_alignment)(struct PE_(r_bin_pe_obj_t)* bin) {
 	return bin->nt_headers->optional_header.SectionAlignment;
 }
 
-struct r_bin_pe_section_t* PE_(r_bin_pe_get_sections)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+struct r_bin_pe_section_t* PE_(r_bin_pe_get_sections)(struct PE_(r_bin_pe_obj_t)* bin) {
 	struct r_bin_pe_section_t *sections = NULL;
 	PE_(image_section_header) *shdr = bin->section_header;
 	int i, sections_count = bin->nt_headers->file_header.NumberOfSections;
 
 	if ((sections = malloc((sections_count + 1) * sizeof(struct r_bin_pe_section_t))) == NULL) {
-		perror("malloc (sections)");
+		perror ("malloc (sections)");
 		return NULL;
 	}
 	for (i = 0; i < sections_count; i++) {
-		memcpy(sections[i].name, shdr[i].Name, PE_IMAGE_SIZEOF_SHORT_NAME);
+		memcpy (sections[i].name, shdr[i].Name, PE_IMAGE_SIZEOF_SHORT_NAME);
 		sections[i].name[PE_IMAGE_SIZEOF_SHORT_NAME-1] = '\0';
 		sections[i].rva = shdr[i].VirtualAddress;
 		sections[i].size = shdr[i].SizeOfRawData;
@@ -645,8 +639,7 @@ struct r_bin_pe_section_t* PE_(r_bin_pe_get_sections)(struct PE_(r_bin_pe_obj_t)
 	return sections;
 }
 
-char* PE_(r_bin_pe_get_subsystem)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+char* PE_(r_bin_pe_get_subsystem)(struct PE_(r_bin_pe_obj_t)* bin) {
 	char *subsystem;
 
 	switch (bin->nt_headers->optional_header.Subsystem) {
@@ -686,60 +679,44 @@ char* PE_(r_bin_pe_get_subsystem)(struct PE_(r_bin_pe_obj_t)* bin)
 	return subsystem;
 }
 
-int PE_(r_bin_pe_is_dll)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+int PE_(r_bin_pe_is_dll)(struct PE_(r_bin_pe_obj_t)* bin) {
 	return bin->nt_headers->file_header.Characteristics & PE_IMAGE_FILE_DLL;
 }
 
-int PE_(r_bin_pe_is_big_endian)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+int PE_(r_bin_pe_is_big_endian)(struct PE_(r_bin_pe_obj_t)* bin) {
 	return bin->nt_headers->file_header.Characteristics & PE_IMAGE_FILE_BYTES_REVERSED_HI;
 }
 
-int PE_(r_bin_pe_is_stripped_relocs)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+int PE_(r_bin_pe_is_stripped_relocs)(struct PE_(r_bin_pe_obj_t)* bin) {
 	return bin->nt_headers->file_header.Characteristics & PE_IMAGE_FILE_RELOCS_STRIPPED;
 }
 
-int PE_(r_bin_pe_is_stripped_line_nums)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+int PE_(r_bin_pe_is_stripped_line_nums)(struct PE_(r_bin_pe_obj_t)* bin) {
 	return bin->nt_headers->file_header.Characteristics & PE_IMAGE_FILE_LINE_NUMS_STRIPPED;
 }
 
-int PE_(r_bin_pe_is_stripped_local_syms)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+int PE_(r_bin_pe_is_stripped_local_syms)(struct PE_(r_bin_pe_obj_t)* bin) {
 	return bin->nt_headers->file_header.Characteristics & PE_IMAGE_FILE_LOCAL_SYMS_STRIPPED;
 }
 
-int PE_(r_bin_pe_is_stripped_debug)(struct PE_(r_bin_pe_obj_t)* bin)
-{
+int PE_(r_bin_pe_is_stripped_debug)(struct PE_(r_bin_pe_obj_t)* bin) {
 	return bin->nt_headers->file_header.Characteristics & PE_IMAGE_FILE_DEBUG_STRIPPED;
 }
 
-void* PE_(r_bin_pe_free)(struct PE_(r_bin_pe_obj_t)* bin)
-{
-	if (!bin)
-		return NULL;
-	if (bin->dos_header)
-		free(bin->dos_header);
-	if (bin->nt_headers)
-		free(bin->nt_headers);
-	if (bin->section_header)
-		free(bin->section_header);
-	if (bin->export_directory)
-		free(bin->export_directory);
-	if (bin->import_directory)
-		free(bin->import_directory);
-	if (bin->delay_import_directory)
-		free(bin->delay_import_directory);
-	if (bin->b)
-		r_buf_free(bin->b);
-	free(bin);
+void* PE_(r_bin_pe_free)(struct PE_(r_bin_pe_obj_t)* bin) {
+	if (!bin) return NULL;
+	free (bin->dos_header);
+	free (bin->nt_headers);
+	free (bin->section_header);
+	free (bin->export_directory);
+	free (bin->import_directory);
+	free (bin->delay_import_directory);
+	r_buf_free (bin->b);
+	free (bin);
 	return NULL;
 }
 
-struct PE_(r_bin_pe_obj_t)* PE_(r_bin_pe_new)(const char* file)
-{
+struct PE_(r_bin_pe_obj_t)* PE_(r_bin_pe_new)(const char* file) {
 	struct PE_(r_bin_pe_obj_t) *bin;
 	ut8 *buf;
 
@@ -758,8 +735,7 @@ struct PE_(r_bin_pe_obj_t)* PE_(r_bin_pe_new)(const char* file)
 	return bin;
 }
 
-struct PE_(r_bin_pe_obj_t)* PE_(r_bin_pe_new_buf)(struct r_buf_t *buf)
-{
+struct PE_(r_bin_pe_obj_t)* PE_(r_bin_pe_new_buf)(struct r_buf_t *buf) {
 	struct PE_(r_bin_pe_obj_t) *bin;
 
 	if (!(bin = malloc(sizeof(struct PE_(r_bin_pe_obj_t)))))
