@@ -61,7 +61,6 @@ static RList* entries(RBinArch *arch) {
 static RList* sections(RBinArch *arch) {
 	RList *ret = NULL;
 	RBinSection *ptr = NULL;
-	struct r_bin_elf_field_t *field = NULL;
 	struct r_bin_elf_section_t *section = NULL;
 	int i;
 
@@ -78,6 +77,8 @@ static RList* sections(RBinArch *arch) {
 		ptr->vsize = section[i].size;
 		ptr->offset = section[i].offset;
 		ptr->rva = section[i].rva;
+		// HACK
+		if (ptr->rva == 0) ptr->rva = section[i].offset;
 		ptr->srwx = 0;
 		if (R_BIN_ELF_SCN_IS_EXECUTABLE (section[i].flags))
 			ptr->srwx |= 1;
@@ -243,7 +244,8 @@ static RBinInfo* info(RBinArch *arch) {
 	free (str);
 	strncpy (ret->rclass, "elf", R_BIN_SIZEOF_STRINGS);
 	ret->bits = Elf_(r_bin_elf_get_bits) (arch->bin_obj);
-	ret->big_endian=Elf_(r_bin_elf_is_big_endian) (arch->bin_obj);
+	ret->big_endian = Elf_(r_bin_elf_is_big_endian) (arch->bin_obj);
+	ret->has_va = Elf_(r_bin_elf_has_va) (arch->bin_obj);
 	ret->dbg_info = 0;
 	if (!Elf_(r_bin_elf_get_stripped) (arch->bin_obj))
 		ret->dbg_info |= 0x04 | 0x08 | 0x10;
@@ -295,8 +297,8 @@ static RBuffer* create(RBin* bin, const ut8 *code, int codelen, const ut8 *data,
 	ut32 baddr;
 	int is_arm = !strcmp (bin->curarch.info->arch, "arm");
 	RBuffer *buf = r_buf_new ();
-	if (is_arm)
-		baddr = 0x40000;
+	// XXX: hardcoded
+	if (is_arm) baddr = 0x40000;
 	else baddr = 0x8048000;
 
 #define B(x,y) r_buf_append_bytes(buf,(const ut8*)x,y)
