@@ -49,9 +49,11 @@ static int config_bigendian_callback(void *user, void *data) {
 static int config_iova_callback(void *user, void *data) {
 	RCore *core = (RCore *) user;
 	RConfigNode *node = (RConfigNode *) data;
-	core->io->va = node->i_value;
-	// reload symbol information
-	r_core_cmd0 (core, ".ia*");
+	if (node->i_value != core->io->va) {
+		core->io->va = node->i_value;
+		// reload symbol information
+		r_core_cmd0 (core, ".ia*");
+	}
 	return R_TRUE;
 }
 
@@ -97,7 +99,7 @@ static int config_cfgdebug_callback(void *user, void *data) {
 	RConfigNode *node = (RConfigNode*) data;
 	if (core && core->io)
 		core->io->debug = node->i_value;
-	if (core->dbg && node->i_value) {
+	if (core && core->dbg && node->i_value) {
 		const char *dbgbackend = r_config_get (core->config, "dbg.backend");
 		r_debug_use (core->dbg, dbgbackend);
 		if (!strcmp (dbgbackend, "bf"))
@@ -203,7 +205,7 @@ static int asm_profile(RConfig *cfg, const char *profile) {
 		r_config_set (cfg, "asm.trace", "false");
 		r_config_set (cfg, "asm.bytes", "false");
 		r_config_set (cfg, "asm.stackptr", "false");
-	} else if (!strcmp(profile, "graph")) {
+	} else if (!strcmp (profile, "graph")) {
 		asm_profile (cfg, "default");
 		r_config_set (cfg, "asm.section", "false");
 		r_config_set (cfg, "asm.bytes", "false");
@@ -213,11 +215,11 @@ static int asm_profile(RConfig *cfg, const char *profile) {
 		r_config_set (cfg, "asm.stackptr", "false");
 		if (r_config_get (cfg, "graph.offset"))
 			r_config_set (cfg, "asm.offset", "true");
-		else   r_config_set (cfg, "asm.offset", "false");
-	} else if (!strcmp(profile, "debug")) {
+		else r_config_set (cfg, "asm.offset", "false");
+	} else if (!strcmp (profile, "debug")) {
 		asm_profile (cfg, "default");
 		r_config_set (cfg, "asm.trace", "true");
-	} else if (!strcmp(profile, "full")) {
+	} else if (!strcmp (profile, "full")) {
 		asm_profile (cfg, "default");
 		r_config_set (cfg, "asm.bytes", "true");
 		r_config_set (cfg, "asm.lines", "true");
@@ -225,7 +227,7 @@ static int asm_profile(RConfig *cfg, const char *profile) {
 		r_config_set (cfg, "asm.lineswide", "true");
 		r_config_set (cfg, "asm.section", "true");
 		r_config_set (cfg, "asm.size", "true");
-	} else if (!strcmp(profile, "simple")) {
+	} else if (!strcmp (profile, "simple")) {
 		asm_profile (cfg, "default");
 		r_config_set (cfg, "asm.bytes", "false");
 		r_config_set (cfg, "asm.lines", "false");
@@ -335,8 +337,8 @@ static int config_asmarch_callback(void *user, void *data) {
 	r_config_set (core->config, "anal.plugin", node->value);
 	if (!r_syscall_setup (core->anal->syscall, node->value,
 			asmos, core->anal->bits)) {
-		eprintf ("asm.arch: Cannot setup syscall '%s/%s' from '%s'\n",
-			node->value, asmos, R2_LIBDIR"/radare2/"R2_VERSION"/syscall");
+		//eprintf ("asm.arch: Cannot setup syscall '%s/%s' from '%s'\n",
+		//	node->value, asmos, R2_LIBDIR"/radare2/"R2_VERSION"/syscall");
 	}
 	//if (!strcmp (node->value, "bf"))
 	//	r_config_set (core->config, "dbg.backend", "bf");
@@ -370,6 +372,14 @@ static int config_asmbits_callback(void *user, void *data) {
 		eprintf ("asm.arch: Cannot setup '%i' bits analysis engine\n", (int)node->i_value);
 	if (core->dbg  && core->anal && core->anal->cur)
 		r_debug_set_arch (core->dbg, core->anal->cur->arch, node->i_value);
+	const char *asmos = r_config_get (core->config, "asm.os");
+	const char *asmarch = r_config_get (core->config, "asm.arch");
+	if (core && core->anal)
+	if (!r_syscall_setup (core->anal->syscall, asmarch,
+			asmos, node->i_value)) {
+		//eprintf ("asm.arch: Cannot setup syscall '%s/%s' from '%s'\n",
+		//	node->value, asmos, R2_LIBDIR"/radare2/"R2_VERSION"/syscall");
+	}
 	return ret;
 }
 
@@ -389,7 +399,11 @@ R_API int r_core_config_init(RCore *core) {
 	char *p;
 	cfg->printf = r_cons_printf;
 
+	//r_config_set (cfg, "dir.opcodes", R_ASM_OPCODES_PATH);
 	r_config_set (cfg, "dir.source", "");
+	r_config_desc (cfg, "dir.source", "Path to find source files");
+	r_config_set (cfg, "dir.magic", R_MAGIC_PATH);
+	r_config_desc (cfg, "dir.magic", "Path to r_magic files");
 	r_config_set (cfg, "dir.plugins", LIBDIR"/radare2/"R2_VERSION"/");
 	r_config_desc (cfg, "dir.plugins", "Path to plugin files to be loaded at startup");
 	/* anal */
@@ -411,6 +425,10 @@ R_API int r_core_config_init(RCore *core) {
 	r_config_desc (cfg, "asm.bits", "Word size in bits at assembler");
 	r_config_set (cfg, "asm.bytes", "true");
 	r_config_desc (cfg, "asm.bytes", "Display the bytes of each instruction");
+	r_config_set (cfg, "asm.flags", "true");
+	r_config_desc (cfg, "asm.bytes", "Show flags in disassembly (pd)");
+	r_config_set (cfg, "asm.size", "false");
+	r_config_desc (cfg, "asm.size", "Show size of opcodes in disassembly (pd)");
 	r_config_set (cfg, "asm.lbytes", "true");
 	r_config_desc (cfg, "asm.lbytes", "Align disasm bytes to left");
 	r_config_set (cfg, "asm.middle", "false"); // jump in the middle because of antidisasm tricks
@@ -449,15 +467,19 @@ R_API int r_core_config_init(RCore *core) {
 	r_config_set (cfg, "asm.linescall", "false");
 	r_config_desc (cfg, "asm.linescall", "Enable call lines");
 	r_config_set_cb (cfg, "asm.os", R_SYS_OS, &config_asmos_callback);
+	r_config_desc (cfg, "asm.os", "Select operating system (kernel) (linux, darwin, w32,..)");
 	r_config_set_cb (cfg, "asm.syntax", "intel", &config_asmsyntax_callback);
 	r_config_desc (cfg, "asm.syntax", "Select assembly syntax");
 	r_config_set_cb (cfg, "asm.profile", "default", &config_asmprofile_callback);
+	r_config_desc (cfg, "asm.profile", "configure disassembler (default, simple, gas, smart, debug, full)");
 #if LIL_ENDIAN
 	r_config_set_cb (cfg, "cfg.bigendian", "false", &config_bigendian_callback);
 #else
 	r_config_set_cb (cfg, "cfg.bigendian", "true", &config_bigendian_callback);
 #endif
+	r_config_desc (cfg, "cfg.bigendian", "Use little (false) or big (true) endiannes\n");
 	r_config_set_cb (cfg, "cfg.debug", "false", &config_cfgdebug_callback);
+	r_config_desc (cfg, "cfg.debug", "set/unset the debugger mode");
 	r_config_set_cb (cfg, "cfg.datefmt", "%d:%m:%Y %H:%M:%S %z", &config_cfgdatefmt_callback);
 	r_config_desc (cfg, "cfg.datefmt", "Date format (%d:%m:%Y %H:%M:%S %z)");
 	r_config_set (cfg, "cfg.fortunes", "true");
@@ -476,13 +498,18 @@ R_API int r_core_config_init(RCore *core) {
 	r_config_set (cfg, "dbg.bep", "loader"); // loader, entry, constructor, main
 	r_config_set_cb (cfg, "dbg.stopthreads", "true", &config_stopthreads_callback);
 	r_config_set_cb (cfg, "dbg.swstep", "false", &config_swstep_callback);
+	r_config_desc (cfg, "dbg.swstep", "If enabled forces the use of software steps (code analysis+breakpoint)");
 	r_config_set_cb (cfg, "dbg.trace", "true", &config_trace_callback);
 	r_config_set_cb (cfg, "dbg.trace.tag", "0xff", &config_tracetag_callback);
 	r_config_set_cb (cfg, "fs.view", "normal", &config_fsview_callback);
 	r_config_desc (cfg, "fs.view", "Set visibility options for filesystems");
 	r_config_set (cfg, "bin.strings", "true");
 	p = r_sys_getenv ("EDITOR");
+#if __WINDOWS__
+	r_config_set (cfg, "cfg.editor", p? p: "notepad");
+#else
 	r_config_set (cfg, "cfg.editor", p? p: "vi");
+#endif
 	r_config_desc (cfg, "cfg.editor", "Select default editor program");
 	free (p);
 	r_config_set (cfg, "cmd.hit", "");
@@ -499,8 +526,11 @@ R_API int r_core_config_init(RCore *core) {
 	r_config_desc (cfg, "cmd.vprompt", "Visual prompt commands");
 	r_config_set (cfg, "cmd.bp", "");
 	r_config_desc (cfg, "cmd.bp", "Command to executed every breakpoint hitted");
+	r_config_set (cfg, "graph.font", "Courier");
+	r_config_desc (cfg, "graph.font", "font to be used by the dot graphs");
 	r_config_set_cb (cfg, "scr.interactive", "true", config_scrint_callback);
 	r_config_set_cb (cfg, "scr.tee", "", config_teefile_callback);
+	r_config_desc (cfg, "scr.tee", "Pipe console output to file if not empty");
 	r_config_set_cb (cfg, "scr.prompt", "true", &config_scrprompt_callback);
 	r_config_set_cb (cfg, "scr.color",
 		(core->print->flags&R_PRINT_FLAGS_COLOR)?"true":"false",
@@ -512,6 +542,8 @@ R_API int r_core_config_init(RCore *core) {
 	r_config_set (cfg, "scr.seek", "");
 	r_config_set_i_cb (cfg, "scr.cols", 16, &config_scrcols_callback);
 	r_config_desc (cfg, "scr.cols", "Configure the number of columns to print");
+	r_config_set (cfg, "search.in", "file");
+	r_config_desc (cfg, "search.in", "Specify search boundaries. (raw, file, section)");
 	r_config_set_i (cfg, "search.kwidx", 0);
 	r_config_desc (cfg, "search.kwidx", "Store last search index count");
 	r_config_set (cfg, "search.flags", "true");
@@ -528,11 +560,10 @@ R_API int r_core_config_init(RCore *core) {
 	r_config_desc (cfg, "search.distance", "Search string distance");
 	r_config_set_i_cb (cfg, "search.align", 0, &config_searchalign_callback);
 	r_config_desc (cfg, "search.align", "Only catch aligned search hits");
-	r_config_set (cfg, "search.asmstr", "true");
-	r_config_desc (cfg, "search.asmstr", "Search string instead of assembly");
 	r_config_set_cb (cfg, "scr.html", "false", &config_scrhtml_callback);
 	r_config_desc (cfg, "scr.html", "If enabled disassembly use HTML syntax");
 	r_config_set_cb (cfg, "io.ffio", "true", &config_ioffio_callback);
+	r_config_desc (cfg, "io.ffio", "fill invalid buffers with 0xff instead of returning error");
 	r_config_set_cb (cfg, "io.va", "true", &config_iova_callback);
 	r_config_desc (cfg, "io.va", "If enabled virtual address layout can be used");
 	r_config_set_cb (cfg, "io.cache", "false", &config_iocache_callback);
@@ -540,6 +571,7 @@ R_API int r_core_config_init(RCore *core) {
 	r_config_set (cfg, "file.path", "");
 	r_config_desc (cfg, "file.path", "Path of current file");
 	r_config_set (cfg, "file.desc", "");
+	r_config_desc (cfg, "file.desc", "User defined file description. Used by projects");
 	r_config_set (cfg, "file.project", "");
 	r_config_desc (cfg, "file.project", "Name of current project");
 	r_config_set (cfg, "file.md5", "");
@@ -549,6 +581,7 @@ R_API int r_core_config_init(RCore *core) {
 	r_config_set (cfg, "file.type", "");
 	r_config_desc (cfg, "file.type", "Type of current file");
 	r_config_set_i (cfg, "magic.depth", 100);
+	r_config_desc (cfg, "magic.depth", "Recursivity depth in magic description strings");
 	r_config_set (cfg, "rap.loop", "true");
 	/* fkeys */
 	r_config_set (cfg, "key.f1", "");
@@ -592,7 +625,6 @@ R_API int r_core_config_init(RCore *core) {
 	config_set("asm.reladdr", "false"); // relative offset
 	config_set("asm.jmpflags", "false");
 
-	config_set("asm.flags", "true");
 	config_set("asm.flagsall", "true");
 	config_set("asm.functions", "true");
 	config_set("asm.lines", "true"); // show left ref lines
@@ -601,7 +633,6 @@ R_API int r_core_config_init(RCore *core) {
 	config_set("asm.trace", "false"); // trace counter
 	config_set("asm.split", "true"); // split code blocks
 	config_set("asm.splitall", "false"); // split code blocks
-	config_set("asm.size", "false"); // opcode size
 	config_set("asm.xrefs", "xrefs");
 
 	// config_set("asm.follow", "");
