@@ -22,8 +22,7 @@ static ut64 num_callback(RNum *userptr, const char *str, int *ok) {
 	*ok = 0;
 	if (str[0]=='$') {
 		*ok = 1;
-		r_anal_op (core->anal, &op, core->offset,
-			core->block, core->blocksize);
+		r_anal_op (core->anal, &op, core->offset, core->block, core->blocksize);
 		switch (str[1]) {
 		case '{':
 			{
@@ -69,7 +68,7 @@ R_API RCore *r_core_new() {
 #define CMDS (sizeof (radare_argv)/sizeof(const char*))
 static const char *radare_argv[] = {
 	"?", 
-	"dH", "ds", "dso", "dsl", "dc", "dd", "dm", "db", "dp", "dr", "dcu",
+	"dH", "ds", "dso", "dsl", "dc", "dd", "dm", "db", "db-", "dp", "dr", "dcu",
 	"S", 
 	"s", "s+", "s++", "s-", "s--", "s*", "sa", "sb", "sr",
 	"!", "!!", 
@@ -81,7 +80,7 @@ static const char *radare_argv[] = {
 	"e", "e-", "e*", "e!",
 	"i", "ii", "iI", "is", "iS", "iz",
 	"q", 
-	"f", "fr", "f-", "f*", "fs", "fS", "fr", "f?",
+	"f", "fl", "fr", "f-", "f*", "fs", "fS", "fr", "fo", "f?",
 	"m", "m*", "ml", "m-", "my", "mg", "md", "mp", "m?",
 	"o", "o-",
 	"x",
@@ -163,15 +162,15 @@ printf ("DIR(%s)\n", path);
 printf ("FILE(%s)\n", p);
 printf ("FILEN %d\n", n);
 #endif
- 			list = p? r_sys_dir (path):NULL;
+ 			list = p? r_sys_dir (path): NULL;
 			if (list) {
 				char buf[4096];
 				r_list_foreach (list, iter, str) {
 					if (*str == '.')
 						continue;
-					if (!*p || !memcmp (str, p, n)) {
+					if (!p || !*p || !memcmp (str, p, n)) {
 						snprintf (buf, sizeof (buf), "%s%s%s",
-							path, strlen(path)>1?"/":"", str);
+							path, strlen (path)>1?"/":"", str);
 						tmp_argv[i++] = strdup (buf);
 						if (i==TMP_ARGV_SZ)
 							break;
@@ -195,6 +194,8 @@ printf ("FILEN %d\n", n);
 		    (!memcmp (line->buffer.data, "b ", 2)) ||
 		    (!memcmp (line->buffer.data, "dcu ", 4)) ||
 		    (!memcmp (line->buffer.data, "/v ", 3)) ||
+		    (!memcmp (line->buffer.data, "db ", 3)) ||
+		    (!memcmp (line->buffer.data, "db- ", 4)) ||
 		    (!memcmp (line->buffer.data, "f ", 2)) ||
 		    (!memcmp (line->buffer.data, "fr ", 3)) ||
 		    (!memcmp (line->buffer.data, "/a ", 3)) ||
@@ -453,7 +454,7 @@ R_API int r_core_prompt_exec(RCore *r) {
 	return ret;
 }
 
-R_API int r_core_block_size(RCore *core, ut32 bsize) {
+R_API int r_core_block_size(RCore *core, int bsize) {
 	int ret = R_FALSE;
 	if (bsize == core->blocksize)
 		return R_FALSE;
@@ -707,13 +708,13 @@ reaccept:
 				r_socket_read_block (c, (ut8*)&bufr, 4);
 				r_mem_copyendian ((ut8*)&i, (ut8 *)bufr, 4, !endian);
 				if (i>0 && i<RMT_MAX) {
-					if ((cmd=malloc(i))) {
+					if ((cmd=malloc (i))) {
 						r_socket_read_block (c, (ut8*)cmd, i);
 						cmd[i] = '\0';
 						eprintf ("len: %d cmd: '%s'\n",
 							i, cmd); fflush(stdout);
 						cmd_output = r_core_cmd_str (core, cmd);
-						free(cmd);
+						free (cmd);
 					} else eprintf ("rap: cannot malloc\n");
 				} else eprintf ("rap: invalid length '%d'\n", i);
 				/* write */
@@ -723,14 +724,14 @@ reaccept:
 					cmd_output = strdup("");
 					cmd_len = 0; 
 				}
-				bufw = malloc(cmd_len + 5);
+				bufw = malloc (cmd_len + 5);
 				bufw[0] = RMT_CMD | RMT_REPLY;
-				r_mem_copyendian((ut8*)bufw+1, (ut8 *)&cmd_len, 4, !endian);
-				memcpy(bufw+5, cmd_output, cmd_len);
+				r_mem_copyendian ((ut8*)bufw+1, (ut8 *)&cmd_len, 4, !endian);
+				memcpy (bufw+5, cmd_output, cmd_len);
 				r_socket_write (c, bufw, cmd_len+5);
 				r_socket_flush (c);
-				free(bufw);
-				free(cmd_output);
+				free (bufw);
+				free (cmd_output);
 				break;
 				}
 			case RMT_WRITE:
@@ -770,18 +771,19 @@ reaccept:
 			case RMT_SYSTEM:
 				// read
 				r_socket_read_block (c, buf, 4);
-				r_mem_copyendian((ut8*)&i, buf, 4, !endian);
+				r_mem_copyendian ((ut8*)&i, buf, 4, !endian);
 				if (i>0&&i<RMT_MAX) {
-					ptr = (ut8 *) malloc(i+6);
+					ptr = (ut8 *) malloc (i+6);
+					if (!ptr) return R_FALSE;
 					ptr[5]='!';
 					r_socket_read_block (c, ptr+6, i);
 					ptr[6+i]='\0';
 					//env_update();
 					//pipe_stdout_to_tmp_file((char*)&buf, (char*)ptr+5);
-					strcpy((char*)buf, "/tmp/.out");
+					strcpy ((char*)buf, "/tmp/.out");
 					pipefd = r_cons_pipe_open ((const char *)buf, 0);
 					//eprintf("SYSTEM(%s)\n", ptr+6);
-					system((const char*)ptr+6);
+					system ((const char*)ptr+6);
 					r_cons_pipe_close (pipefd);
 					{
 						FILE *fd = fopen((char*)buf, "r");
@@ -790,33 +792,36 @@ reaccept:
 							eprintf("Cannot open tmpfile\n");
 							i = -1;
 						} else {
-							fseek(fd, 0, SEEK_END);
-							i = ftell(fd);
-							fseek(fd, 0, SEEK_SET);
-							free(ptr);
-							ptr = (ut8 *) malloc(i+5);
-							fread(ptr+5, i, 1, fd);
+							fseek (fd, 0, SEEK_END);
+							i = ftell (fd);
+							fseek (fd, 0, SEEK_SET);
+							free (ptr);
+							ptr = (ut8 *) malloc (i+5);
+							fread (ptr+5, i, 1, fd);
 							ptr[i+5]='\0';
-							fclose(fd);
+							fclose (fd);
 						}
 					}
 					{
 					char *out = r_file_slurp ((char*)buf, &i);
-					free(ptr);
+					free (ptr);
 					//eprintf("PIPE(%s)\n", out);
-					ptr = (ut8 *) malloc(i+5);
+					ptr = (ut8 *) malloc (i+5);
 					if (ptr) {
 						memcpy (ptr+5, out, i);
-						free(out);
+						free (out);
 					}
 					}
 					//unlink((char*)buf);
 				}
 
+				if (!ptr) ptr = (ut8 *) malloc (5); // malloc for 5 byets? c'mon!
+				if (!ptr) return R_FALSE;
+				
 				// send
 				ptr[0] = (RMT_SYSTEM | RMT_REPLY);
 				r_mem_copyendian ((ut8*)ptr+1, (ut8*)&i, 4, !endian);
-				if (i<0)i=0;
+				if (i<0) i = 0;
 				r_socket_write (c, ptr, i+5);
 				r_socket_flush (c);
 				eprintf ("REPLY SENT (%d) (%s)\n", i, ptr+5);
