@@ -36,7 +36,7 @@ R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int l
 	char *line = NULL, *comment, *opstr, *osl = NULL; // old source line
 	char *refline = NULL;
 	RAsmOp asmop;
-	RAnalOp analop;
+	RAnalOp analop = {0};
 	RFlagItem *flag;
 	RMetaItem *mi;
 	ut64 dest = UT64_MAX;
@@ -56,6 +56,7 @@ R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int l
 #warning asm.dwarf is now marked as experimental and disabled
 	int show_dwarf = 0; // r_config_get_i (core->config, "asm.dwarf");
 	int show_linescall = r_config_get_i (core->config, "asm.linescall");
+	int show_size = r_config_get_i (core->config, "asm.size");
 	int show_trace = r_config_get_i (core->config, "asm.trace");
 	int linesout = r_config_get_i (core->config, "asm.linesout");
 	int adistrick = r_config_get_i (core->config, "asm.middle"); // TODO: find better name
@@ -76,7 +77,6 @@ R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int l
 	int ocols = 0;
 	int lcols = 0;
 	
-
 	if (show_lines) ocols += 10;
 	if (show_offset) ocols += 14;
 	lcols = ocols+2;
@@ -191,8 +191,9 @@ R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int l
 		}
 		// TODO : line analysis must respect data types! shouldnt be interpreted as code
 		ret = r_asm_disassemble (core->assembler, &asmop, buf+idx, len-idx);
-		if (ret<1) {
+		if (ret<1) { // XXX: move to r_asm_disassemble ()
 			ret = 1;
+			asmop.inst_len = 1;
 			//eprintf ("** invalid opcode at 0x%08"PFMT64x" **\n",
 			//	core->assembler->pc + ret);
 			lastfail = 1;
@@ -205,9 +206,9 @@ R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int l
 		if (core->inc == 0)
 			core->inc = ret;
 
-		if (lastfail)
-			memset (&analop, 0, sizeof (analop));
-		else r_anal_op (core->anal, &analop, at, buf+idx, (int)(len-idx));
+		r_anal_op_fini (&analop);
+		if (!lastfail)
+			r_anal_op (core->anal, &analop, at, buf+idx, (int)(len-idx));
 		{
 			RAnalValue *src;
 			switch (analop.type) {
@@ -274,7 +275,7 @@ R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int l
 							refi->type==R_ANAL_REF_TYPE_CALL?"CODE (CALL)":"DATA", refi->addr,
 							fun?fun->name:"unk");
 				}
-				r_list_destroy (xrefs);
+				r_list_free (xrefs);
 			}
 		}
 		if (adistrick)
@@ -315,11 +316,12 @@ R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int l
 				else r_cons_printf ("%s:\n", flag->name);
 			}
 		}
-		if (show_lines && line) {
+		if (show_lines && line)
 			r_cons_strcat (line);
-		}
 		if (show_offset)
 			printoffset (at, show_color, (at==dest));
+		if (show_size)
+			r_cons_printf ("%d ", analop.length);
 		if (show_trace) {
 			RDebugTracepoint *tp = r_debug_trace_get (core->dbg, at);
 			r_cons_printf ("%02x:%04x ", tp?tp->times:0, tp?tp->count:0);
@@ -379,7 +381,7 @@ R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int l
 			r_cons_printf ("*");
 		else r_cons_printf (" ");
 		if (show_bytes) {
-			char *str, pad[64];
+			char *str = NULL, pad[64];
 			char extra[64];
 			strcpy (extra, " ");
 			flag = NULL; // HACK
@@ -428,7 +430,7 @@ R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int l
 					r_cons_printf (" %s %s %s"Color_RESET, pad, str, extra);
 				else r_cons_printf (" %s %s %s", pad, str, extra);
 			}
-			if (!show_color) free (str);
+			free (str);
 		}
 
 		if (show_color) {
@@ -451,6 +453,9 @@ R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int l
 			case R_ANAL_OP_TYPE_SWI:
 				r_cons_printf (Color_MAGENTA);
 				break;
+			case R_ANAL_OP_TYPE_TRAP:
+				r_cons_printf (Color_BRED);
+				break;
 			case R_ANAL_OP_TYPE_RET:
 				r_cons_printf (Color_RED);
 				break;
@@ -515,17 +520,19 @@ R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int l
 			RAsmOp ao; /* disassemble for the vm .. */
 			int os = core->assembler->syntax;
 			r_asm_set_syntax (core->assembler, R_ASM_SYNTAX_INTEL);
-			ret = r_asm_disassemble (core->assembler, &ao, buf+idx, len-idx);
+			r_asm_disassemble (core->assembler, &ao, buf+idx, len-idx);
 			r_asm_set_syntax (core->assembler, os);
 		}
 
 		if (!r_anal_cc_update (core->anal, &cc, &analop)) {
 			if (show_functions) {
 				char *ccstr = r_anal_cc_to_string (core->anal, &cc);
-				if (show_color)
-					r_cons_printf ("\n%s%s   "Color_TURQOISE"; %s"Color_RESET, pre, refline, ccstr);
-				else r_cons_printf ("\n%s%s    ; %s", pre, refline, ccstr);
-				free (ccstr);
+				if (ccstr) {
+					if (show_color)
+						r_cons_printf ("\n%s%s   "Color_TURQOISE"; %s"Color_RESET, pre, refline, ccstr);
+					else r_cons_printf ("\n%s%s    ; %s", pre, refline, ccstr);
+					free (ccstr);
+				}
 			}
 			r_anal_cc_reset (&cc);
 		}
