@@ -120,19 +120,23 @@ R_API int r_core_visual_trackflags(RCore *core) {
 		case 'a':
 			switch (menu) {
 			case 0: // new flag space
+				r_cons_show_cursor (R_TRUE);
 				r_line_set_prompt ("add flagspace: ");
 				strcpy (cmd, "fs ");
 				if (r_cons_fgets (cmd+3, sizeof (cmd)-4, 0, NULL) > 0) {
 					r_core_cmd (core, cmd, 0);
 					r_cons_set_raw (1);
+					r_cons_show_cursor (R_FALSE);
 				}
 				break;
 			case 1: // new flag
+				r_cons_show_cursor (R_TRUE);
 				r_line_set_prompt ("add flag: ");
 				strcpy (cmd, "f ");
 				if (r_cons_fgets (cmd+2, sizeof (cmd)-3, 0, NULL) > 0) {
 					r_core_cmd (core, cmd, 0);
 					r_cons_set_raw (1);
+					r_cons_show_cursor (R_FALSE);
 				}
 				break;
 			}
@@ -163,6 +167,7 @@ R_API int r_core_visual_trackflags(RCore *core) {
 		case 'r':
 			if (menu == 1) {
 				int len;
+				r_cons_show_cursor (R_TRUE);
 				r_cons_set_raw (0);
 				// TODO: use r_flag_rename or wtf?..fr doesnt uses this..
 				snprintf (cmd, sizeof (cmd), "fr %s ", fs2);
@@ -173,6 +178,7 @@ R_API int r_core_visual_trackflags(RCore *core) {
 					cmd[0]='\0';
 				r_core_cmd (core, cmd, 0);
 				r_cons_set_raw (1);
+				r_cons_show_cursor (R_FALSE);
 			}
 			break;
 		case 'P':
@@ -215,6 +221,7 @@ R_API int r_core_visual_trackflags(RCore *core) {
 			r_cons_any_key ();
 			break;
 		case ':':
+			r_cons_show_cursor (R_TRUE);
 			r_cons_set_raw (0);
 			cmd[0]='\0';
 			r_line_set_prompt (":> ");
@@ -223,6 +230,7 @@ R_API int r_core_visual_trackflags(RCore *core) {
 			//line[strlen(line)-1]='\0';
 			r_core_cmd (core, cmd, 1);
 			r_cons_set_raw (1);
+			r_cons_show_cursor (R_FALSE);
 			if (cmd[0])
 				r_cons_any_key ();
 			//cons_gotoxy(0,0);
@@ -233,6 +241,148 @@ R_API int r_core_visual_trackflags(RCore *core) {
 	return R_TRUE;
 }
 
+R_API int r_core_visual_comments (RCore *core) {
+	char *str, cmd[512], *p = NULL;
+	int mode = 0;
+	int delta = 7;
+	int i, ch, option = 0;
+	int format = 0;
+	int found = 0;
+	ut64 from, size;
+	RListIter *iter;
+	RAnalFcn *fcn;
+	RMetaItem *d;
+
+	for (;;) {
+		r_cons_gotoxy (0, 0);
+		r_cons_clear ();
+		r_cons_printf ("Comments:\n");
+
+		i = 0;
+		found = 0;
+		mode = 0;
+		r_list_foreach (core->anal->meta->data, iter, d) {
+			str = r_str_unscape (d->str);
+			if (str) {
+				if (d->type=='s') /* Ignore strings, there are in trackflags */
+					continue;
+				if ((i>=option-delta) && ((i<option+delta)||((option<delta)&&(i<(delta<<1))))) {
+					r_str_sanitize (str);
+					if (option==i) {
+						mode = 0;
+						found = 1;
+						from = d->from;
+						size = d->size;
+						p = str;
+						r_cons_printf("  >  %s\n", str);
+					} else {
+						r_cons_printf("     %s\n", str);
+						free (str);
+					}
+				}
+				i++;
+			}
+		}
+		if (!found) {
+			option--;
+			continue;
+		}
+		r_list_foreach (core->anal->fcns, iter, fcn) {
+			if ((i>=option-delta) && ((i<option+delta)||((option<delta)&&(i<(delta<<1))))) {
+				if (option==i) {
+					mode = 1;
+					from = fcn->addr;
+					size = fcn->size;
+				}
+				r_cons_printf("  %c  %s\n", (option==i)?'>':' ', fcn->name);
+			}
+			i++;
+		}
+
+		switch (format) {
+			case 0: sprintf (cmd, "px @ 0x%"PFMT64x":64", from); core->printidx = 0; break;
+			case 1: sprintf (cmd, "pd 12 @ 0x%"PFMT64x":64", from); core->printidx = 1; break;
+			case 2: sprintf (cmd, "ps @ 0x%"PFMT64x":64", from); core->printidx = 5; break;
+			default: format = 0; continue;
+		}
+		if (*cmd) r_core_cmd (core, cmd, 0);
+
+		r_cons_flush ();
+		ch = r_cons_readchar ();
+		ch = r_cons_arrow_to_hjkl (ch); // get ESC+char, return 'hjkl' char
+		switch (ch) {
+		case 'a':
+			//TODO
+			break;
+		case 'e':
+			//TODO
+			break;
+		case 'd':
+			if (mode == 0) {
+				if (p)
+					r_meta_del (core->anal->meta, R_META_TYPE_ANY, from, size, p);
+			} else {
+				r_anal_fcn_del (core->anal, from);
+			}
+			break;
+		case 'P':
+			if (--format<0)
+				format = MAX_FORMAT;
+			break;
+		case 'p':
+			format++;
+			break;
+		case 'J':
+			option += 10;
+			break;
+		case 'j':
+			option++;
+			break;
+		case 'k':
+			if (--option<0)
+				option = 0;
+			break;
+		case 'K':
+			option-=10;
+			if (option<0)
+				option = 0;
+			break;
+		case 'l':
+		case ' ':
+		case '\r':
+		case '\n':
+			sprintf (cmd, "s 0x%"PFMT64x, from);
+			r_core_cmd (core, cmd, 0);
+			if (p)
+				free (p);
+			return R_TRUE;
+		case 'q':
+			if (p)
+				free (p);
+			return R_TRUE;
+		case '?':
+		case 'h':
+			r_cons_clear00 ();
+			r_cons_printf (
+			"\nVT: Visual Comments/Anal help:\n\n"
+			" q     - quit menu\n"
+			" j/k   - down/up keys\n"
+			" h/b   - go back\n"
+			" l/' ' - accept current selection\n"
+			" a/d/e - add/delete/edit comment/anal symbol\n"
+			" p/P   - rotate print format\n");
+			r_cons_flush ();
+			r_cons_any_key ();
+			break;
+		}
+		if (p) {
+			free (p);
+			p = NULL;
+		}
+	}
+	return R_TRUE;
+}
+
 static void config_visual_hit_i(RCore *core, const char *name, int delta) {
 	struct r_config_node_t *node;
 	node = r_config_node_get (core->config, name);
@@ -259,11 +409,13 @@ static void config_visual_hit(RCore *core, const char *name, int editor) {
 		} else {
 			// FGETS AND SO
 			r_cons_printf ("New value (old=%s): \n", node->value);
+			r_cons_show_cursor (R_TRUE);
 			r_cons_flush ();
 			r_cons_set_raw (0);
 			r_line_set_prompt (":> ");
 			r_cons_fgets (buf, sizeof (buf)-1, 0, 0);
 			r_cons_set_raw (1);
+			r_cons_show_cursor (R_FALSE);
 			node->value = r_str_dup (node->value, buf);
 		}
 	}
@@ -411,6 +563,7 @@ R_API void r_core_visual_config(RCore *core) {
 			r_cons_any_key ();
 			break;
 		case ':':
+			r_cons_show_cursor (R_TRUE);
 			r_cons_set_raw(0);
 /* WTF READLINE?? WE DONT USE THAT!! */
 #if HAVE_LIB_READLINE
@@ -430,6 +583,7 @@ R_API void r_core_visual_config(RCore *core) {
 			r_core_cmd (core, cmd, 1);
 #endif
 			r_cons_set_raw (1);
+			r_cons_show_cursor (R_FALSE);
 			if (cmd[0])
 				r_cons_any_key ();
 			//r_cons_gotoxy(0,0);
@@ -642,10 +796,12 @@ R_API void r_core_visual_mounts (RCore *core) {
 						file = r_fs_open (core->fs, path);
 						if (file) {
 							r_fs_read (core->fs, file, 0, file->size);
+							r_cons_show_cursor (R_TRUE);
 							r_cons_set_raw (0);
 							r_line_set_prompt ("Dump path (ej: /tmp/file): ");
 							r_cons_fgets (buf, sizeof (buf)-1, 0, 0);
 							r_cons_set_raw (1);
+							r_cons_show_cursor (R_FALSE);
 							r_file_dump (buf, file->data, file->size);
 							r_fs_close (core->fs, file);
 							r_cons_printf ("Done\n");
@@ -674,10 +830,12 @@ R_API void r_core_visual_mounts (RCore *core) {
 				r_cons_any_key ();
 				break;
 			case ':':
+				r_cons_show_cursor (R_TRUE);
 				r_cons_set_raw (0);
 				r_line_set_prompt (":> ");
 				r_cons_fgets (buf, sizeof (buf)-1, 0, 0);
 				r_cons_set_raw (1);
+				r_cons_show_cursor (R_FALSE);
 				r_core_cmd (core, buf, 1);
 				r_cons_any_key ();
 				break;
@@ -828,6 +986,7 @@ R_API void r_core_visual_anal(RCore *core) {
 			case 0:
 eprintf ("TODO: Add new function manually\n");
 /*
+				r_cons_show_cursor (R_TRUE);
 				r_cons_set_raw (R_FALSE);
 				r_line_set_prompt ("Address: ");
 				if (!r_cons_fgets (old, sizeof (old), 0, NULL)) break;
@@ -846,6 +1005,7 @@ eprintf ("TODO: Add new function manually\n");
 				//XXX sprintf(cmd, "CF %lld @ 0x%08llx", size, addr);
 				// XXX r_core_cmd0(core, cmd);
 				r_cons_set_raw (R_TRUE);
+				r_cons_show_cursor (R_FALSE);
 */
 				break;
 			case 1:
@@ -853,6 +1013,7 @@ eprintf ("TODO: Add new function manually\n");
 			}
 			break;
 		case 'm':
+			r_cons_show_cursor (R_TRUE);
 			r_cons_set_raw (R_FALSE);
 			r_line_set_prompt ("New name: ");
 			if (!r_cons_fgets (old, sizeof (old), 0, NULL)) break;
@@ -860,6 +1021,7 @@ eprintf ("TODO: Add new function manually\n");
 			function_rename (core, addr, old);
 		
 			r_cons_set_raw (R_TRUE);
+			r_cons_show_cursor (R_FALSE);
 			break;
 		case 'd':
 			switch (level) {
