@@ -1045,8 +1045,11 @@ static RList *r_debug_native_pids(int pid) {
 			fd = open (cmdline, O_RDONLY);
 			if (fd == -1)
 				continue;
-			read (fd, cmdline, 1024);
-			cmdline[1023] = '\0';
+			if (read (fd, cmdline, sizeof (cmdline))==-1) {
+				close (fd);
+				continue;
+			}
+			cmdline[sizeof (cmdline)-1] = '\0';
 			ptr = strstr (cmdline, "PPid: ");
 			if (ptr) {
 				int ppid = atoi (ptr+6);
@@ -1093,7 +1096,7 @@ static RList *r_debug_native_threads(int pid) {
 	return w32_thread_list (pid, list);
 #elif __APPLE__
 #if __arm__                 
-	#define OSX_PC state.r15
+	#define OSX_PC state.pc
 #elif __POWERPC__
 	#define OSX_PC state.srr0
 #elif __x86_64__
@@ -1309,7 +1312,8 @@ static int r_debug_native_reg_write(RDebug *dbg, int type, const ut8* buf, int s
 	// XXX use switch or so
 	if (type == R_REG_TYPE_DRX) {
 #if __KFBSD__
-		return (0 == ptrace (PT_SETDBREGS, pid, (caddr_t)buf, sizeof (struct dbreg)));
+		return (0 == ptrace (PT_SETDBREGS, dbg->pid,
+			(caddr_t)buf, sizeof (struct dbreg)));
 #elif __linux__
 // XXX: this android check is only for arm
 #ifndef __ANDROID__
@@ -1869,9 +1873,9 @@ static void addr_to_string(struct sockaddr_storage *ss, char *buffer, int buflen
 	case AF_LOCAL:
 		sun = (struct sockaddr_un *)ss;
 		if (strlen (sun->sun_path) == 0)
-			strlcpy(buffer, "-", buflen);
+			strlcpy (buffer, "-", buflen);
 		else
-			strlcpy(buffer, sun->sun_path, buflen);
+			strlcpy (buffer, sun->sun_path, buflen);
 		break;
 
 	case AF_INET:
@@ -1882,16 +1886,16 @@ static void addr_to_string(struct sockaddr_storage *ss, char *buffer, int buflen
 
 	case AF_INET6:
 		sin6 = (struct sockaddr_in6 *)ss;
-		if (inet_ntop(AF_INET6, &sin6->sin6_addr, buffer2,
-		    sizeof(buffer2)) != NULL)
-			snprintf(buffer, buflen, "%s.%d", buffer2,
-			    ntohs(sin6->sin6_port));
+		if (inet_ntop (AF_INET6, &sin6->sin6_addr, buffer2,
+		    sizeof (buffer2)) != NULL)
+			snprintf (buffer, buflen, "%s.%d", buffer2,
+			    ntohs (sin6->sin6_port));
 		else
-			strlcpy(buffer, "-", sizeof(buffer));
+			strlcpy (buffer, "-", sizeof(buffer));
 		break;
 
 	default:
-		strlcpy(buffer, "", buflen);
+		strlcpy (buffer, "", buflen);
 		break;
 	}
 }
@@ -2024,6 +2028,41 @@ static RList *r_debug_desc_native_list (int pid) {
 	return ret;
 }
 
+#if __APPLE__
+vm_prot_t unix_prot_to_darwin(int prot) {
+        return ((prot&1<<4)?VM_PROT_READ:0 |
+                (prot&1<<2)?VM_PROT_WRITE:0 |
+                (prot&1<<1)?VM_PROT_EXECUTE:0);
+}
+#endif
+static int r_debug_native_map_protect (RDebug *dbg, ut64 addr, int size, int perms) {
+#if __WINDOWS__
+        DWORD old;
+	HANDLE hProcess = tid2handler (dbg->pid, dbg->tid);
+	// TODO: align pointers
+        return VirtualProtectEx (WIN32_PI (hProcess), (LPVOID)(UINT)addr, size, perms, &old);
+#elif __APPLE__
+	int ret;
+	// TODO: align pointers
+	ret = vm_protect (pid_to_task (dbg->tid),
+			(vm_address_t)addr,
+			(vm_size_t)size,
+			(boolean_t)0, /* maximum protection */
+			perms); //unix_prot_to_darwin (perms));
+	if (ret != KERN_SUCCESS) {
+		printf("vm_protect failed\n");
+		return R_TRUE;
+	}
+	return R_FALSE;
+#elif __linux__
+#warning mprotect not implemented for this Linux.. contribs are welcome. use r_egg here?
+	return R_FALSE;
+#else
+#warning mprotect not implemented for this platform
+	return R_FALSE;
+#endif
+}
+
 static int r_debug_desc_native_open (const char *path) {
 	return 0;
 }
@@ -2069,6 +2108,7 @@ struct r_debug_plugin_t r_debug_plugin_native = {
 	.reg_read = r_debug_native_reg_read,
 	.reg_write = (void *)&r_debug_native_reg_write,
 	.map_get = r_debug_native_map_get,
+	.map_protect = r_debug_native_map_protect,
 	.breakpoint = r_debug_native_bp,
 };
 
