@@ -26,20 +26,20 @@ int main(int argc, char **argv) {
 	size = r_io_size (io);
 
 	/* read bytes */
-	buf = (ut8*) malloc(size);
+	buf = (ut8*) malloc (size);
 	if (buf == NULL) {
-		printf ("Too big file\n");
+		printf ("Big too file\n");
 		r_io_close (io, fd);
 		r_io_free (io);
 		return 1;
 	}
 
-	memset(buf, 0, size);
-	r_io_read(io, buf, size);
-	printf("----\n%s\n----\n", buf);
+	memset (buf, 0, size);
+	r_io_read (io, buf, size);
+	printf ("----\n%s\n----\n", buf);
 
-	printf("file size = %"PFMT64d"\n", size);
-	printf("CRC32: 0x%08x\n", r_hash_crc32(buf, size));
+	printf ("file size = %"PFMT64d"\n", size);
+	printf ("CRC32: 0x%08x\n", r_hash_crc32(buf, size));
 
 	{
 		struct r_hash_t *ctx;
@@ -47,20 +47,20 @@ int main(int argc, char **argv) {
 		int i;
 		//r_hash_init(&ctx, R_TRUE, R_HASH_ALL);
 		ctx = r_hash_new (R_TRUE, R_HASH_ALL);
-		c = r_hash_do_md5(ctx, buf, size);
-		printf("MD5: ");
-		for(i=0;i<R_HASH_SIZE_MD5;i++) { printf("%02x", c[i]); }
-		printf("\n");
+		c = r_hash_do_md5 (ctx, buf, size);
+		printf ("MD5: ");
+		for (i=0;i<R_HASH_SIZE_MD5;i++) { printf("%02x", c[i]); }
+		printf ("\n");
 
-		c = r_hash_do_sha1(ctx, buf, size);
-		printf("SHA1: ");
-		for(i=0;i<R_HASH_SIZE_SHA1;i++) { printf("%02x", c[i]); }
-		printf("\n");
+		c = r_hash_do_sha1 (ctx, buf, size);
+		printf ("SHA1: ");
+		for (i=0;i<R_HASH_SIZE_SHA1;i++) { printf("%02x", c[i]); }
+		printf ("\n");
 
-		c = r_hash_do_sha256(ctx, buf, size);
-		printf("SHA256: ");
-		for(i=0;i<R_HASH_SIZE_SHA256;i++) { printf("%02x", c[i]); }
-		printf("\n");
+		c = r_hash_do_sha256 (ctx, buf, size);
+		printf ("SHA256: ");
+		for (i=0;i<R_HASH_SIZE_SHA256;i++) { printf("%02x", c[i]); }
+		printf ("\n");
 		r_hash_free (ctx);
 	}
 
