@@ -11,6 +11,16 @@
 #include <r_util.h>
 #include <r_syscall.h>
 
+// TODO: save memory2 : fingerprints must be pointers to a buffer
+// containing a dupped file in memory
+
+/* save memory:
+   bb_has_ops=1 -> 600M 
+   bb_has_ops=0 -> 350MB
+*/
+#define R_ANAL_BB_HAS_OPS 0
+
+// TODO: Remove this define? /cc @nibble_ds
 #define VERBOSE_ANAL if(0)
 
 /* meta */
@@ -154,19 +164,23 @@ enum {
 
 typedef struct r_anal_t {
 	int bits;
-	int lineswidth;
+	int lineswidth; // wtf
 	int big_endian;
-	int split;
+	int split; // used only from core
 	void *user;
 	RList *fcns;
+	RListRange *fcnstore;
 	RList *refs;
 	RList *vartypes;
 	RMeta *meta;
 	RReg *reg;
 	RSyscall *syscall;
 	struct r_anal_op_t *queued;
+	int diff_ops;
+	double diff_thbb;
+	double diff_thfcn;
 	RIOBind iob;
-	struct r_anal_ctx_t *ctx;
+	//struct r_anal_ctx_t *ctx;
 	struct r_anal_plugin_t *cur;
 	struct list_head anals; // TODO: Reimplement with RList
 } RAnal;
@@ -224,6 +238,7 @@ enum {
 typedef struct r_anal_diff_t {
 	int type;
 	ut64 addr;
+	double dist;
 	char *name;
 } RAnalDiff;
 
@@ -239,7 +254,9 @@ typedef struct r_anal_bb_t {
 	int traced;
 	ut8 *fingerprint;
 	RAnalDiff *diff;
+#if R_ANAL_BB_HAS_OPS
 	RList *ops;
+#endif
 	RAnalCond *cond;
 } RAnalBlock;
 
@@ -279,7 +296,8 @@ enum {
 	R_ANAL_FCN_TYPE_FCN = 1,
 	R_ANAL_FCN_TYPE_LOC = 2,
 	R_ANAL_FCN_TYPE_SYM = 4,
-	R_ANAL_FCN_TYPE_IMP = 8
+	R_ANAL_FCN_TYPE_IMP = 8,
+	R_ANAL_FCN_TYPE_ROOT = 16  /* matching flag */
 };
 
 #define R_ANAL_VARSUBS 32
@@ -306,6 +324,11 @@ typedef struct r_anal_fcn_t {
 	RList *xrefs;
 } RAnalFcn;
 
+typedef struct r_anal_fcn_store_t {
+	RHashTable64 *h;
+	RList *l;
+} RAnalFcnStore;
+
 typedef struct r_anal_var_access_t {
 	ut64 addr;
 	int set;
@@ -375,10 +398,20 @@ typedef struct r_anal_plugin_t {
 	struct list_head list;
 } RAnalPlugin;
 
+
 #ifdef R_API
+/* --------- */ /* REFACTOR */ /* ---------- */
+R_API RListRange* r_listrange_new ();
+R_API void r_listrange_free(RListRange *s);
+R_API void r_listrange_add(RListRange *s, RAnalFcn *f);
+R_API void r_listrange_del(RListRange *s, RAnalFcn *f);
+R_API void r_listrange_resize(RListRange *s, RAnalFcn *f, int newsize);
+R_API RAnalFcn *r_listrange_find_in_range(RListRange* s, ut64 addr);
+R_API RAnalFcn *r_listrange_find_root(RListRange* s, ut64 addr);
+/* --------- */ /* REFACTOR */ /* ---------- */
 /* anal.c */
 R_API RAnal *r_anal_new();
-R_API RAnal *r_anal_free(RAnal *r);
+R_API void r_anal_free(RAnal *r);
 R_API void r_anal_set_user_ptr(RAnal *anal, void *user);
 R_API int r_anal_add(RAnal *anal, struct r_anal_plugin_t *foo);
 R_API int r_anal_list(RAnal *anal);
@@ -389,12 +422,12 @@ R_API int r_anal_set_big_endian(RAnal *anal, int boolean);
 R_API char *r_anal_strmask (RAnal *anal, const char *data);
 R_API void r_anal_trace_bb(RAnal *anal, ut64 addr);
 R_API RAnalFcn *r_anal_get_fcn_at(RAnal *anal, ut64 addr);
-R_API RList *r_anal_get_fcns(RAnal *anal);
+#define r_anal_get_fcns(x) x->fcns
 
 /* bb.c */
 R_API RAnalBlock *r_anal_bb_new();
 R_API RList *r_anal_bb_list_new();
-R_API void r_anal_bb_free(void *bb);
+R_API void r_anal_bb_free(RAnalBlock *bb);
 R_API int r_anal_bb(RAnal *anal, RAnalBlock *bb,
 		ut64 addr, ut8 *buf, ut64 len, int head);
 R_API RAnalBlock *r_anal_bb_from_offset(RAnal *anal, ut64 off);
@@ -402,6 +435,7 @@ R_API RAnalBlock *r_anal_bb_from_offset(RAnal *anal, ut64 off);
 /* op.c */
 R_API RAnalOp *r_anal_op_new();
 R_API void r_anal_op_free(void *op);
+R_API void r_anal_op_fini(RAnalOp *op);
 R_API RList *r_anal_op_list_new();
 R_API int r_anal_op(RAnal *anal, RAnalOp *op, ut64 addr,
 		const ut8 *data, int len);
@@ -411,6 +445,7 @@ R_API char *r_anal_op_to_string(RAnal *anal, RAnalOp *op);
 R_API RAnalFcn *r_anal_fcn_new();
 R_API RAnalFcn *r_anal_fcn_find(RAnal *anal, ut64 addr, int type);
 R_API RList *r_anal_fcn_list_new();
+R_API int r_anal_fcn_insert(RAnal *anal, RAnalFcn *fcn);
 R_API void r_anal_fcn_free(void *fcn);
 R_API int r_anal_fcn(RAnal *anal, RAnalFcn *fcn, ut64 addr,
 		ut8 *buf, ut64 len, int reftype);
@@ -425,6 +460,10 @@ R_API int r_anal_fcn_overlap_bb(RAnalFcn *fcn, RAnalBlock *bb);
 R_API RAnalVar *r_anal_fcn_get_var(RAnalFcn *fs, int num, int dir);
 R_API char *r_anal_fcn_to_string(RAnal *a, RAnalFcn* fs);
 R_API int r_anal_fcn_from_string(RAnal *a, RAnalFcn *f, const char *_str);
+#define r_anal_fcn_get_refs(x) x->refs
+#define r_anal_fcn_get_xrefs(x) x->xrefs
+#define r_anal_fcn_get_vars(x) x->vars
+#define r_anal_fcn_get_bbs(x) x->bbs
 
 /* ref.c */
 R_API RAnalRef *r_anal_ref_new();
@@ -458,8 +497,11 @@ R_API RAnalVarAccess *r_anal_var_access_get(RAnal *anal, RAnalVar *var, ut64 fro
 
 #define R_ANAL_THRESHOLDFCN 0.7F
 #define R_ANAL_THRESHOLDBB 0.7F
+
 /* diff.c */
 R_API RAnalDiff *r_anal_diff_new();
+R_API void r_anal_diff_setup(RAnal *anal, int doops, double thbb, double thfcn);
+R_API void r_anal_diff_setup_i(RAnal *anal, int doops, int thbb, int thfcn);
 R_API void* r_anal_diff_free(RAnalDiff *diff);
 R_API int r_anal_diff_fingerprint_bb(RAnal *anal, RAnalBlock *bb);
 R_API int r_anal_diff_fingerprint_fcn(RAnal *anal, RAnalFcn *fcn);
@@ -479,7 +521,8 @@ R_API void r_anal_value_free(RAnalValue *value);
 
 R_API RAnalCond *r_anal_cond_new();
 R_API RAnalCond *r_anal_cond_new_from_op(RAnalOp *op);
-#define r_anal_cond_free(x) free(x);
+R_API void r_anal_cond_fini (RAnalCond *c);
+R_API void r_anal_cond_free (RAnalCond *c);
 R_API char *r_anal_cond_to_string(RAnalCond *cond);
 R_API int r_anal_cond_eval (RAnal *anal, RAnalCond *cond);
 R_API RAnalCond *r_anal_cond_new_from_string(const char *str);
@@ -520,6 +563,9 @@ R_API int r_meta_list(RMeta *m, int type);
 R_API void r_meta_item_free(void *_item);
 R_API RMetaItem *r_meta_item_new(int type);
 
+R_API int r_anal_fcn_xref_add (RAnal *anal, RAnalFcn *fcn, ut64 at, ut64 addr, int type);
+R_API int r_anal_fcn_xref_del (RAnal *anal, RAnalFcn *fcn, ut64 at, ut64 addr, int type);
+
 /* plugin pointers */
 extern RAnalPlugin r_anal_plugin_csr;
 extern RAnalPlugin r_anal_plugin_avr;
@@ -531,6 +577,8 @@ extern RAnalPlugin r_anal_plugin_java;
 extern RAnalPlugin r_anal_plugin_mips;
 extern RAnalPlugin r_anal_plugin_dalvik;
 extern RAnalPlugin r_anal_plugin_sh;
+extern RAnalPlugin r_anal_plugin_sparc;
+extern RAnalPlugin r_anal_plugin_bf;
 
 #endif
 #endif
