@@ -281,8 +281,10 @@ R_API char *r_line_readline() {
 	memset (&buf, 0, sizeof buf);
 	r_cons_set_raw (1);
 
+//r_cons_gotoxy()
 	if (I.echo) {
-		printf ("\r%s", I.prompt);
+		r_cons_clear_line();
+		eprintf ("\x1b[0K\r%s", I.prompt);
 		fflush (stdout);
 	}
 	for (;;) {
@@ -306,7 +308,8 @@ R_API char *r_line_readline() {
 			return NULL; //I.buffer.data;
 		buf[0] = ch;
 		
-//		printf("\x1b[K\r");
+		if (I.echo)
+			r_cons_clear_line();
 		columns = r_cons_get_size (NULL)-2;
 		if (columns<1)
 			columns = 40;
@@ -331,7 +334,7 @@ R_API char *r_line_readline() {
 			break;
 		case 3: // ^C 
 			if (I.echo)
-				printf ("^C\n");
+				eprintf ("^C\n");
 			I.buffer.index = I.buffer.length = 0;
 			*I.buffer.data = '\0';
 			goto _end;
