@@ -31,10 +31,12 @@ R_API int r_buf_set_bits(RBuffer *b, int bitoff, int bitsize, ut64 value) {
 	return R_FALSE;
 }
 
-R_API int r_buf_set_bytes(RBuffer *b, ut8 *buf, int length) {
+R_API int r_buf_set_bytes(RBuffer *b, const ut8 *buf, int length) {
 	if (b->buf)
 		free (b->buf);
-	if (!(b->buf = malloc (length)))
+	if (length<0)
+		return R_FALSE;
+	if (!(b->buf = malloc (length+1)))
 		return R_FALSE;
 	memcpy (b->buf, buf, length);
 	b->length = length;
@@ -50,6 +52,9 @@ R_API int r_buf_prepend_bytes(RBuffer *b, const ut8 *buf, int length) {
 	return R_TRUE;
 }
 
+// TODO: R_API void r_buf_insert_bytes() // with shift
+// TODO: R_API void r_buf_write_bytes() // overwrite
+
 R_API char *r_buf_to_string(RBuffer *b) {
 	char *s;
 	if (!b) return strdup ("");
@@ -99,6 +104,14 @@ R_API int r_buf_append_ut64(RBuffer *b, ut64 n) {
 	return R_TRUE;
 }
 
+R_API int r_buf_append_buf(RBuffer *b, RBuffer *a) {
+	if (!(b->buf = realloc (b->buf, b->length+a->length)))
+		return R_FALSE;
+	memcpy (b->buf+b->length, a->buf, a->length);
+	b->length += a->length;
+	return R_TRUE;
+}
+
 static int r_buf_cpy(RBuffer *b, ut64 addr, ut8 *dst, const ut8 *src, int len, int write) {
 	int end;
 	addr = (addr==R_BUF_CUR)? b->cur: addr-b->base;
@@ -174,6 +187,7 @@ R_API void r_buf_deinit(struct r_buf_t *b) {
 }
 
 R_API void r_buf_free(struct r_buf_t *b) {
+	if (!b) return;
 	r_buf_deinit (b);
 	free (b);
 }
