@@ -26,6 +26,13 @@ R_API boolt r_file_exist(const char *str) {
 	return (S_ISREG (buf.st_mode))? R_TRUE: R_FALSE;
 }
 
+R_API int r_file_size(const char *str) {
+	struct stat buf;
+	if (stat (str, &buf)==-1)
+		return 0;
+	return buf.st_size;
+}
+
 R_API char *r_file_abspath(const char *file) {
 	char *ret = NULL;
 	char *cwd = r_sys_getdir ();
@@ -148,11 +155,13 @@ R_API char *r_file_slurp_random_line(const char *file) {
 	if (str) {
 		gettimeofday (&tv,NULL);
 		srand (getpid()+tv.tv_usec);
-		for(i=0; str[i]; i++)
+		for (i=0; str[i]; i++)
 			if (str[i]=='\n')
 				lines++;
+		if (lines<1)
+			return NULL;
 		lines = (rand()%lines);
-		for(i=0; str[i]&&lines; i++)
+		for (i=0; str[i] && lines; i++)
 			if (str[i]=='\n')
 				lines--;
 		ptr = str+i;
@@ -222,7 +231,7 @@ R_API boolt r_file_rm(const char *file) {
 R_API RMmap *r_file_mmap (const char *file, boolt rw) {
 	RMmap *m = NULL;
 #if __WINDOWS__
-	int fd = open (file, 0);
+	int fd = open (file, O_BINARY);
 #else
 	int fd = open (file, rw? O_RDWR: O_RDONLY);
 #endif
@@ -306,7 +315,7 @@ R_API int r_file_mkstemp (const char *prefix, char **oname) {
 	char *name = malloc (1024);
 #if __WINDOWS__
 	if (GetTempFileName (path, prefix, 0, name))
-		h = open (name, O_RDWR|O_EXCL);
+		h = open (name, O_RDWR|O_EXCL|O_BINARY);
 	else h = -1;
 #else
 	h = snprintf (name, 1024, "%s/%sXXXXXX", path, prefix);
