@@ -75,10 +75,9 @@ R_API boolt r_list_delete_data (RList *list, void *ptr) {
 
 R_API void r_list_delete (RList *list, RListIter *iter) {
 	r_list_split_iter (list, iter);
-	if (list->free && iter->data) {
+	if (list->free && iter->data)
 		list->free (iter->data);
-		iter->data = NULL;
-	}
+	iter->data = NULL;
 	free (iter);
 }
 
@@ -88,6 +87,24 @@ R_API RList *r_list_new() {
 	return list;
 }
 
+/* remove all elements of a list */
+R_API void r_list_purge (RList *list) {
+	RListIter *it;
+	if (list) {
+		it = list->head;
+		while (it) {
+			RListIter *next = it->n;
+			r_list_delete (list, it);
+			it = next;
+		//	free (it);
+		}
+		list->head = list->tail = NULL;
+	}
+	//free (list);
+}
+
+// same as purge.. but without a correct name.. need refactoring
+// TODO: rename to r_list_purge() or find a better name
 R_API void r_list_destroy (RList *list) {
 	RListIter *it;
 	if (list) {
@@ -119,7 +136,7 @@ R_API RListIter *r_list_item_new (void *data) {
 
 R_API RListIter *r_list_append(RList *list, void *data) {
 	RListIter *new = NULL;
-	if (data) {
+	if (list && data) {
 		new = R_NEW (RListIter);
 		if (list->tail)
 			list->tail->n = new;
@@ -283,14 +300,14 @@ R_API void *r_list_get_by_string(RList *list, int off, const char *str) {
 	return NULL;
 }
 
-R_API boolt r_list_contains (RList *list, void *p) {
+R_API RListIter *r_list_contains (RList *list, void *p) {
 	void *q;
 	RListIter *iter;
 	r_list_foreach (list, iter, q) {
 		if (p == q)
-			return R_TRUE;
+			return iter;
 	}
-	return R_FALSE;
+	return NULL;
 }
 
 #if TEST
