@@ -6,6 +6,7 @@ cd `dirname $PWD/$0` ; cd ..
 . ./sys/CONFIG
 cat sys/CONFIG
 
+export PYTHON
 export PYTHON_VERSION
 export PYTHON_CONFIG
 echo "Using PYTHON_VERSION ${PYTHON_VERSION}"
@@ -15,8 +16,9 @@ echo
 
 cd r2-bindings
 ./configure --prefix=/usr --enable-devel --enable=python
+sudo make install-vapi
 cd python
 make clean
-make
+make PYTHON=${PYTHON}
 [ ! "$1" = --no-install ] && \
-	sudo make install PYTHON_VERSION=${PYTHON_VERSION}
+	sudo make install PYTHON=${PYTHON} PYTHON_VERSION=${PYTHON_VERSION}
